@@ -1,3 +1,23 @@
+//===========================================================================
+//
+// Copyright (C) 2006-2009 Yves Renard, Julien Pommier, Jeremie Lasry.
+//
+// This file is a part of GETFEM++
+//
+// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+// under  the  terms  of the  GNU  Lesser General Public License as published
+// by  the  Free Software Foundation;  either version 2.1 of the License,  or
+// (at your option) any later version.
+// This program  is  distributed  in  the  hope  that it will be useful,  but
+// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+// License for more details.
+// You  should  have received a copy of the GNU Lesser General Public License
+// along  with  this program;  if not, write to the Free Software Foundation,
+// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+//
+//===========================================================================
+
 #include "crack_bilaplacian.h"
 #include "getfem/getfem_regular_meshes.h"
 #include "getfem/getfem_assembling.h" /* import assembly methods (and norms comp.) */
@@ -211,47 +231,47 @@ bool bilaplacian_crack_problem::solve_moment(plain_vector &U) {
       //cout << "MORTAR_BOUNDARY_IN: " << mesh.region(MORTAR_BOUNDARY_IN) << "\n";
       //cout << "MORTAR_BOUNDARY_OUT: " << mesh.region(MORTAR_BOUNDARY_OUT) << "\n";
       
-      // an optional treatment : creating a representation of the enrichment area     
-      getfem::mesh_fem mf_enrich(mesh);
-      getfem::pfem pf_mef = getfem::classical_fem(mesh.trans_of_convex(mesh.convex_index().first_true()), 1 );
-      mf_enrich.set_finite_element(mesh.convex_index(), pf_mef) ;
-      std::vector<scalar_type> UU(mf_enrich.nb_dof()) ;
-      std::fill(UU.begin(), UU.end() ,0.) ;
-      cout << "exporting the enrichment zone: \n" ;
-      for (dal::bv_visitor i(cvlist_in_area) ; !i.finished() ; ++i){ 
-	  for (unsigned int j = 0 ; j < mf_enrich.ind_dof_of_element(i).size() ; ++j )  
-	  UU[mf_enrich.ind_dof_of_element(i)[j]] = 1. ;         
-      }
-      
-      cout << "exporting enrichment to " << "enrichment_zone.vtk" << "..\n";
-      getfem::vtk_export exp("enrichment_zone.vtk", false);
-      exp.exporting(mf_enrich); 
-      exp.write_point_data(mf_enrich, UU, "enrichment");
-      cout << "export done, you can view the data file with (for example)\n"
-	"mayavi -d enrichment_zone.vtk -f "
-	"WarpScalar -m BandedSurfaceMap -m Outline\n";
+//       // an optional treatment : creating a representation of the enrichment area     
+//       getfem::mesh_fem mf_enrich(mesh);
+//       getfem::pfem pf_mef = getfem::classical_fem(mesh.trans_of_convex(mesh.convex_index().first_true()), 1 );
+//       mf_enrich.set_finite_element(mesh.convex_index(), pf_mef) ;
+//       std::vector<scalar_type> UU(mf_enrich.nb_dof()) ;
+//       std::fill(UU.begin(), UU.end() ,0.) ;
+//       cout << "exporting the enrichment zone: \n" ;
+//       for (dal::bv_visitor i(cvlist_in_area) ; !i.finished() ; ++i){ 
+// 	  for (unsigned int j = 0 ; j < mf_enrich.ind_dof_of_element(i).size() ; ++j )  
+// 	  UU[mf_enrich.ind_dof_of_element(i)[j]] = 1. ;         
+//       }
+//       
+//       cout << "exporting enrichment to " << "enrichment_zone.vtk" << "..\n";
+//       getfem::vtk_export exp("enrichment_zone.vtk", false);
+//       exp.exporting(mf_enrich); 
+//       exp.write_point_data(mf_enrich, UU, "enrichment");
+//       cout << "export done, you can view the data file with (for example)\n"
+// 	"mayavi -d enrichment_zone.vtk -f "
+// 	"WarpScalar -m BandedSurfaceMap -m Outline\n";
 	
-      // Another optional treatment :
-      // Searching the elements that are both crossed by the crack
-      // and with one of their faces which constitutes a part of the 
-      // boundary between the enriched zone and the rest of the domain.
-      getfem::mesh_region &boundary = mesh.region(MORTAR_BOUNDARY_IN);
-      unsigned int cpt = 0 ;
-      for (dal::bv_visitor i(cvlist_in_area); !i.finished(); ++i) {
-         if (mls.is_convex_cut(i)){
-	    // Among the faces of the convex, we search if some are
-	    // part of the boundary
-	    cpt = 0 ;
-	    for (unsigned j=0; j < mesh.structure_of_convex(i) ->nb_faces(); ++j) {
-	        if (boundary.is_in(i,j))
-		   cpt += 1;
-	    }
-	    if (cpt) {
-               cout << "\n The convex number " << i << " is crossed by the crack :\n" ;
-	       cout << "  it has : " << cpt << " face(s) among the boundary.\n \n " ;
-	    }
-	 }
-      }
+//       // Another optional treatment :
+//       // Searching the elements that are both crossed by the crack
+//       // and with one of their faces which constitutes a part of the 
+//       // boundary between the enriched zone and the rest of the domain.
+//       getfem::mesh_region &boundary = mesh.region(MORTAR_BOUNDARY_IN);
+//       unsigned int cpt = 0 ;
+//       for (dal::bv_visitor i(cvlist_in_area); !i.finished(); ++i) {
+//          if (mls.is_convex_cut(i)){
+// 	    // Among the faces of the convex, we search if some are
+// 	    // part of the boundary
+// 	    cpt = 0 ;
+// 	    for (unsigned j=0; j < mesh.structure_of_convex(i) ->nb_faces(); ++j) {
+// 	        if (boundary.is_in(i,j))
+// 		   cpt += 1;
+// 	    }
+// 	    if (cpt) {
+//                cout << "\n The convex number " << i << " is crossed by the crack :\n" ;
+// 	       cout << "  it has : " << cpt << " face(s) among the boundary.\n \n " ;
+// 	    }
+// 	 }
+//       }
  
     }
     break ;
@@ -260,36 +280,7 @@ bool bilaplacian_crack_problem::solve_moment(plain_vector &U) {
 	break ;  
 	}
   mesh.write_to_file("toto.mesh");
-  
-  if (0) {  // printing the type of each dof
-    unsigned Q = mf_u().get_qdim();
-    for (unsigned d=0; d < mf_u().nb_dof(); d += Q) {
-      printf("dof %4d @ %+6.2f:%+6.2f: ", d, 
-             mf_u().point_of_dof(d)[0], mf_u().point_of_dof(d)[1]);
-      
-      
-      const getfem::mesh::ind_cv_ct cvs = mf_u().convex_to_dof(d);
-      for (unsigned i=0; i < cvs.size(); ++i) {
-        size_type cv = cvs[i];
-        //if (pm_cvlist.is_in(cv)) flag1 = true; else flag2 = true;
-        
-        getfem::pfem pf = mf_u().fem_of_element(cv);
-        unsigned ld = unsigned(-1);
-        for (unsigned dd = 0; dd < mf_u().nb_dof_of_element(cv); dd += Q) {
-          if (mf_u().ind_dof_of_element(cv)[dd] == d) {
-            ld = dd/Q; break;
-          }
-        }
-        if (ld == unsigned(-1)) {
-          cout << "DOF " << d << "NOT FOUND in " << cv << " BUG BUG\n";
-        } else {
-          printf(" %3d:%.16s", int(cv), name_of_dof(pf->dof_types().at(ld)).c_str());
-        }
-      }
-      printf("\n");
-    }
-  }
-  
+
   cout << "Number of dof for u: " << mf_u().nb_dof() << endl;
 
   // Bilaplacian brick.
@@ -391,10 +382,11 @@ getfem::mdbrick_normal_derivative_source_term<>
 }
 
 void bilaplacian_crack_problem::compute_error_beta(plain_vector &U) {
+  GMM_ASSERT1(!mf_rhs.is_reduced(), "To be adapted");
   std::vector<scalar_type> V(mf_rhs.nb_dof());
   getfem::interpolation(mf_u(), mf_rhs, U, V);
   for (size_type i = 0; i < mf_rhs.nb_dof(); ++i)
-    V[i] -= sol_beta(mf_rhs.point_of_dof(i));
+    V[i] -= sol_beta(mf_rhs.point_of_basic_dof(i));
   cout.precision(8);
 //   cout  << "L2 error = " << getfem::asm_L2_norm(mim, mf_rhs, V)  << endl
 //         << "H1 error = " << getfem::asm_H1_norm(mim, mf_rhs, V)  << endl
@@ -407,7 +399,7 @@ cout << getfem::asm_H1_norm(mim, mf_rhs, V) << " " << getfem::asm_H2_norm(mim, m
 cout << getfem::asm_H1_semi_norm(mim, mf_rhs, V) << " " <<getfem::asm_H2_semi_norm(mim, mf_rhs, V) << endl ;
   if(PARAM.real_value("NORM_EXACT") != 0. ){
     for (size_type i = 0; i < mf_rhs.nb_dof(); ++i)
-        V[i] = sol_beta(mf_rhs.point_of_dof(i));
+        V[i] = sol_beta(mf_rhs.point_of_basic_dof(i));
     cout << "display exact solution: \n" ; 
     cout << "L2 norm: " <<  getfem::asm_L2_norm(mim, mf_rhs, V) << endl ;
     cout << "H1 norm: " <<  getfem::asm_H1_norm(mim, mf_rhs, V) << endl ;
