@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //===========================================================================
 //
-// Copyright (C) 2002-2008 Yves Renard.
+// Copyright (C) 2002-2009 Yves Renard.
 //
 // This file is a part of GETFEM++
 //
@@ -20,15 +20,12 @@
 //===========================================================================
 
 /**
- * Dynamic friction in linear elasticity.
- *
- * This program is used to check that getfem++ is working. This is also 
- * a good example of use of Getfem++.
+   Dynamic friction in linear elasticity with a mixed formulation in
+   displacement/velocity.
  */
 
-
-#include "getfem/getfem_assembling.h" /* import assembly methods (and norms comp.) */
-#include "getfem/getfem_export.h"   /* export functions (save solution in a file)  */
+#include "getfem/getfem_assembling.h"
+#include "getfem/getfem_export.h"
 #include "getfem/getfem_import.h"
 #include "getfem/getfem_regular_meshes.h"
 #include "getfem/getfem_Coulomb_friction.h"
@@ -72,13 +69,13 @@ struct friction_problem {
   scalar_type friction_coef; /* friction coefficient.                        */
 
   scalar_type residual;      /* max residual for the iterative solvers       */
-  size_type N, noisy;
+  size_type N, noisy, contact_type;
   scalar_type T, dt, r;
   scalar_type init_vert_pos, init_vert_speed, hspeed, dtexport;
   scalar_type Dirichlet_ratio;
-  bool  dxexport, Dirichlet;
+  bool  dxexport, Dirichlet, compare;
 
-  std::string datafilename;
+  std::string datafilename, refrootname, INTEGRATION;
   bgeot::md_param PARAM;
 
   void solve(void);
@@ -93,8 +90,7 @@ struct friction_problem {
 void friction_problem::init(void) {
   std::string FEM_TYPE_U  = PARAM.string_value("FEM_TYPE_U","FEM name");
   std::string FEM_TYPE_V  = PARAM.string_value("FEM_TYPE_V","FEM name");
-  std::string INTEGRATION = PARAM.string_value("INTEGRATION",
-					       "Name of integration method");
+  INTEGRATION = PARAM.string_value("INTEGRATION","Name of integration method");
   cout << "FEM_TYPE_U = "  << FEM_TYPE_U << "\n";
   cout << "FEM_TYPE_V = "  << FEM_TYPE_V << "\n";
   cout << "INTEGRATION = " << INTEGRATION << "\n";
@@ -107,7 +103,12 @@ void friction_problem::init(void) {
   mesh.optimize_structure();
 
   datafilename = PARAM.string_value("ROOTFILENAME","Base name of data files.");
-  residual = PARAM.real_value("RESIDUAL");
+  refrootname = PARAM.string_value("REFROOTNAME",
+				   "Base name of reference files.");
+  compare = (PARAM.int_value("COMPARE") != 0);
+  contact_type = PARAM.int_value("CONTACT_TYPE");
+  cout << "contact_type = " << contact_type << endl;
+  residual = PARAM.real_value("RESIDUAL", "residual for iterative methods");
   if (residual == 0.) residual = 1e-10;
 
   mu = PARAM.real_value("MU", "Lam coefficient mu");
@@ -128,7 +129,7 @@ void friction_problem::init(void) {
   dtexport = dt * double(int(dtexport / dt + 0.5));
  
   r = PARAM.real_value("R", "augmentation parameter");
-  noisy = (PARAM.int_value("NOISY", "verbosity of iterative methods") != 0);
+  noisy = PARAM.int_value("NOISY", "verbosity of iterative methods");
   init_vert_pos = PARAM.real_value("INIT_VERT_POS", "initial position");
   init_vert_speed = PARAM.real_value("INIT_VERT_SPEED","initial speed");
   hspeed = PARAM.real_value("FOUNDATION_HSPEED","initial speed");
@@ -366,9 +367,12 @@ void friction_problem::solve(void) {
   cout << "Number of dof for u: " << mf_u.nb_dof() << endl;
   cout << "Number of dof for v: " << mf_v.nb_dof() << endl;
 
+  GMM_ASSERT1(!mf_u.is_reduced(), "To be adapted");
+
   size_type ref_dof = 0;
   for (size_type i = 1; i < mf_u.nb_dof(); ++i)
-    if (mf_u.point_of_dof(i)[N-1] < mf_u.point_of_dof(ref_dof)[N-1])
+    if (mf_u.point_of_basic_dof(i)[N-1]
+	< mf_u.point_of_basic_dof(ref_dof)[N-1])
       ref_dof = i;
 
   // Linearized elasticity brick.
@@ -392,24 +396,49 @@ void friction_problem::solve(void) {
   // Dirichlet condition brick.
   gmm::clear(F);
   for (size_type i = 0; i < nb_dof_rhs; ++i)
-    F[(i+1)*N-1] = Dirichlet_ratio * mf_rhs.point_of_dof(i)[N-1];
+    F[(i+1)*N-1] = Dirichlet_ratio * mf_rhs.point_of_basic_dof(i)[N-1];
   getfem::mdbrick_Dirichlet<> DIRICHLET(VOL_F, DIRICHLET_BOUNDARY);
   DIRICHLET.rhs().set(mf_rhs, F);
   
   // contact condition for Lagrange elements
-  dal::bit_vector cn = mf_u.dof_on_set(CONTACT_BOUNDARY);
+  dal::bit_vector cn, dn = mf_u.basic_dof_on_region(CONTACT_BOUNDARY);
+  // cout << "dn = " << dn << endl;
+  for (dal::bv_visitor cv(mesh.convex_index()); !cv.finished(); ++cv)
+    for (size_type i = 0; i < mf_u.nb_basic_dof_of_element(cv); ++i) {
+      size_type j = mf_u.ind_basic_dof_of_element(cv)[i];
+      if (dn.is_in(j) && j % N == 0) {
+        switch (contact_type) {
+	case 0 :
+	  cn.add(j); break;
+	case 1 :
+	  {
+	    base_node pt = mf_u.fem_of_element(cv)->node_of_dof(0,i/N);
+	    bool state = true;
+	    for (unsigned k = 0; k < pt.size(); ++k)
+	      if (gmm::abs(pt[k]) > 1E-12 && gmm::abs(pt[k]-1.0) > 1E-12)
+		state = false;
+	    if (state) cn.add(j);
+	  }
+	  break;
+	default : GMM_ASSERT1(false, "Bad contact type.");
+	}
+      }
+  }
+
   cout << "cn = " << cn << endl;
-  sparse_matrix BN(cn.card()/N, mf_u.nb_dof());
-  sparse_matrix BT((N-1)*cn.card()/N, mf_u.nb_dof());
-  plain_vector gap(cn.card()/N);
-  size_type jj = 0;
-  for (dal::bv_visitor i(cn); !i.finished(); ++i)
-    if (i % N == 0) {
-      BN(jj, i+N-1) = -1.;
-      gap[jj] = mf_u.point_of_dof(i)[N-1];
-      for (size_type k = 0; k < N-1; ++k) BT((N-1)*jj+k, i+k) = 1.;
-      ++jj;
-    }
+  sparse_matrix BN(cn.card(), mf_u.nb_dof());
+  sparse_matrix BT((N-1)*cn.card(), mf_u.nb_dof());
+  plain_vector gap(cn.card());
+  size_type jj = 0, lower_dof_u = 0, lower_dof_s = 0;
+  scalar_type heighmin = 1E10;
+  for (dal::bv_visitor i(cn); !i.finished(); ++i) {
+    BN(jj, i+N-1) = -1.;
+    gap[jj] = mf_u.point_of_basic_dof(i)[N-1];
+    if (gap[jj] < heighmin)
+      { heighmin = gap[jj]; lower_dof_u = i+N-1; lower_dof_s = jj; }
+    for (size_type k = 0; k < N-1; ++k) BT((N-1)*jj+k, i+k) = 1.;
+    ++jj;
+  }
 
   getfem::mdbrick_Coulomb_friction<>
     // FRICTION(DIRICHLET, BN, gap, friction_coef, BT);
@@ -435,11 +464,12 @@ void friction_problem::solve(void) {
   scalar_type t(0), t_export(dtexport), beta_(0);
   scalar_type alpha_(0);
 
+  GMM_ASSERT1(!mf_u.is_reduced(), "To be adapted");
   // Initial conditions (U0, V0)
   gmm::clear(U0); gmm::clear(V0); gmm::clear(LT0);
   for (size_type i=0; i < mf_u.nb_dof(); ++i)
     if ((i % N) == 0) { 
-      U0[i+N-1] = Dirichlet ? (Dirichlet_ratio * mf_u.point_of_dof(i)[N-1])
+      U0[i+N-1] = Dirichlet ? (Dirichlet_ratio * mf_u.point_of_basic_dof(i)[N-1])
 	: init_vert_pos;
       HSPEED[i] = hspeed;
     }
@@ -472,8 +502,7 @@ void friction_problem::solve(void) {
     exp->serie_add_object("vonmisessteps");
   }
   
-  std::ofstream fileout1("time", std::ios::out);   
-  std::ofstream fileout2("energy", std::ios::out);
+  std::ofstream fileout((datafilename + ".data").c_str(), std::ios::out);
  
   scalar_type Einit = (0.5*gmm::vect_sp(ELAS.get_K(), U0, U0)
 		       + 0.5 * gmm::vect_sp(DYNAMIC.get_C(), V0, V0)
@@ -497,8 +526,7 @@ void friction_problem::solve(void) {
     DYNAMIC.set_DFV(DFV);
     
     iter.init();
-    gmm::default_newton_line_search ls(size_type(-1), 4.0/3.0,
-				       1.0/20.0, 9.0/10.0, 1.1);
+    gmm::default_newton_line_search ls;
     getfem::standard_solve(MS, DYNAMIC, iter,
 			   getfem::default_linear_solver(DYNAMIC), ls);
     
@@ -533,13 +561,14 @@ void friction_problem::solve(void) {
     
     cout << "t = " << t << " J1 = " << J1 << " Jdemi = " << Jdemi;
     cout << " (st " << nbst << ", sl " << nbsl << ")" << endl;
+
+    fileout << t-dt/2 << "    \t" << Jdemi  << "    \t" << LN1[lower_dof_s]
+	    << "    \t" << Udemi[lower_dof_u] << "\n";
     
     gmm::copy(U1, U0); gmm::copy(V1, V0);  J0 = J1;
     gmm::copy(LN1, LN0); gmm::copy(LT1, LT0);
     if (dxexport && t >= t_export-dt/20.0) {
       
-      fileout1 << t << "\n";
-      fileout2 << J1   << "\n";	
       
       exp->write_point_data(mf_u, U0);
       exp->serie_add_object("deformationsteps");
@@ -551,6 +580,39 @@ void friction_problem::solve(void) {
       t_export += dtexport;
     }
   }
+
+  // save the last time step for convergence test.
+  mf_u.write_to_file(datafilename + ".mfu", true);
+  gmm::vecsave(datafilename + ".U", U0);
+
+  if (compare) {
+    getfem::mesh m_ref;
+    m_ref.read_from_file(refrootname + ".mfu");
+    getfem::mesh_fem mf_ref(m_ref);
+    mf_ref.read_from_file(refrootname + ".mfu");
+    plain_vector Uref(mf_ref.nb_dof());
+    gmm::vecload(refrootname + ".U", Uref);
+    
+    plain_vector U(mf_ref.nb_dof());
+
+    getfem::interpolation(mf_u, mf_ref, U0, U, true);
+    getfem::mesh_im mim_ref(m_ref);
+    getfem::pintegration_method ppi = 
+      getfem::int_method_descriptor(INTEGRATION);
+    mim_ref.set_integration_method(m_ref.convex_index(), ppi);
+    
+    cout << "To ref L2 ERROR:"
+	 << getfem::asm_L2_dist(mim_ref, mf_ref, U, mf_ref, Uref) << endl;
+    
+    cout << "To ref H1 ERROR:"
+	 << getfem::asm_H1_dist(mim_ref, mf_ref, U, mf_ref, Uref) << endl;
+
+    cout << "H1 Norm:"
+	 << getfem::asm_H1_norm(mim_ref, mf_ref, Uref) << endl;
+    
+	
+  }
+
 }
   
 /**************************************************************************/
