@@ -72,9 +72,9 @@ struct hyperbolic_problem {
   size_type N, noisy, scheme;
   scalar_type T, dt, r;
   scalar_type dirichlet_val, dtexport;
-  bool  dxexport;
+  bool  dxexport, compare;
 
-  std::string datafilename;
+  std::string datafilename, refrootname, INTEGRATION;
   bgeot::md_param PARAM;
 
   void solve(void);
@@ -89,8 +89,7 @@ struct hyperbolic_problem {
 void hyperbolic_problem::init(void) {
   std::string FEM_TYPE_U  = PARAM.string_value("FEM_TYPE_U","FEM name");
   std::string FEM_TYPE_V  = PARAM.string_value("FEM_TYPE_V","FEM name");
-  std::string INTEGRATION = PARAM.string_value("INTEGRATION",
-					       "Name of integration method");
+  INTEGRATION = PARAM.string_value("INTEGRATION","Name of integration method");
   cout << "FEM_TYPE_U = "  << FEM_TYPE_U << "\n";
   cout << "FEM_TYPE_V = "  << FEM_TYPE_V << "\n";
   cout << "INTEGRATION = " << INTEGRATION << "\n";
@@ -103,6 +102,9 @@ void hyperbolic_problem::init(void) {
   mesh.optimize_structure();
 
   datafilename = PARAM.string_value("ROOTFILENAME","Base name of data files.");
+  refrootname = PARAM.string_value("REFROOTNAME",
+				   "Base name of reference files.");
+  compare = (PARAM.int_value("COMPARE") != 0);
   residual = PARAM.real_value("RESIDUAL");
   if (residual == 0.) residual = 1e-10;
 
@@ -115,7 +117,7 @@ void hyperbolic_problem::init(void) {
   dtexport = dt * double(int(dtexport / dt + 0.5));
  
   r = PARAM.real_value("R", "augmentation parameter");
-  noisy = (PARAM.int_value("NOISY", "verbosity of iterative methods") != 0);
+  noisy = PARAM.int_value("NOISY", "verbosity of iterative methods");
   scheme = PARAM.int_value("SCHEME", "scheme");
 
   /* set the finite element on the mf_u */
@@ -329,14 +331,15 @@ void hyperbolic_problem::solve(void) {
   cout << "Number of dof for v: " << mf_v.nb_dof() << endl;
 
   // Choosing a reference dof.
+  GMM_ASSERT1(!mf_u.is_reduced(), "To be adapted");
   size_type ref_dof = 0, ref_mult = size_type(-1);
   base_node P(N); gmm::fill(P, 0.5);
   for (size_type i = 1; i < mf_u.nb_dof(); ++i)
-    if (gmm::vect_dist2(mf_u.point_of_dof(i), P)
-	< gmm::vect_dist2(mf_u.point_of_dof(ref_dof), P))
+    if (gmm::vect_dist2(mf_u.point_of_basic_dof(i), P)
+	< gmm::vect_dist2(mf_u.point_of_basic_dof(ref_dof), P))
       ref_dof = i;
   cout << "ref_dof = " << ref_dof << " point = "
-       << mf_u.point_of_dof(ref_dof) << endl;
+       << mf_u.point_of_basic_dof(ref_dof) << endl;
 
   // Linearized elasticity brick.
   getfem::mdbrick_generic_elliptic<> ELAS(mim, mf_u);
@@ -362,13 +365,12 @@ void hyperbolic_problem::solve(void) {
   DIRICHLET.rhs().set(mf_rhs, F);
   
   // Contact condition for Lagrange elements
-  dal::bit_vector cn, dn = mf_u.dof_on_set(DIRICHLET_BOUNDARY), pcn;
+  dal::bit_vector cn, dn = mf_u.basic_dof_on_region(DIRICHLET_BOUNDARY);
   for (size_type i = 0; i < mf_u.nb_dof(); ++i)
     if (!dn.is_in(i)) {
 
-      if (mesh.points().search_node(mf_u.point_of_dof(i)) != size_type(-1)) {
-	pcn.add(i); cn.add(i);
-      }
+      if (mesh.points().search_node(mf_u.point_of_basic_dof(i)) != size_type(-1))
+	cn.add(i);
     }
 
  
@@ -442,7 +444,7 @@ void hyperbolic_problem::solve(void) {
   
 
   size_type nitexp = 0;
-  while (t <= T) {
+  while (t < T-1e-10) {
 
     switch(scheme) {
     case 0 : beta_ = 1./dt; alpha_ = 1.; break;
@@ -460,8 +462,7 @@ void hyperbolic_problem::solve(void) {
     DYNAMIC.set_DFV(DFV);
     
     iter.init();
-    gmm::default_newton_line_search ls(size_type(-1), 4.0/3.0,
-				       1.0/20.0, 9.0/10.0, 1.1);
+    gmm::default_newton_line_search ls;
     getfem::standard_solve(MS, DYNAMIC, iter,
 			   getfem::default_linear_solver(DYNAMIC), ls);
     
@@ -492,7 +493,7 @@ void hyperbolic_problem::solve(void) {
     t += dt; 
     scalar_type umin = dirichlet_val;
     size_type nbco = 0;
-    for (dal::bv_visitor i(pcn); !i.finished(); ++i)
+    for (dal::bv_visitor i(cn); !i.finished(); ++i)
       { umin = std::min(Udemi[i], umin); if (Udemi[i] < 1E-4) nbco++; }
     cout << "t = " << t << " J1 = " << J1 << " Jdemi = " << Jdemi
 	 << " nbcontact : " << nbco << " umin = " << umin << endl;
@@ -523,6 +524,35 @@ void hyperbolic_problem::solve(void) {
 	
       }
     }
+
+  }
+
+  // save the last time step for convergence test.
+  mf_u.write_to_file(datafilename + ".mfu", true);
+  gmm::vecsave(datafilename + ".U", U0);
+  
+  if (compare) {
+    getfem::mesh m_ref;
+    m_ref.read_from_file(refrootname + ".mfu");
+    getfem::mesh_fem mf_ref(m_ref);
+    mf_ref.read_from_file(refrootname + ".mfu");
+    plain_vector Uref(mf_ref.nb_dof());
+    gmm::vecload(refrootname + ".U", Uref);
+    
+    plain_vector U(mf_ref.nb_dof());
+    
+    getfem::interpolation(mf_u, mf_ref, U0, U, true);
+    getfem::mesh_im mim_ref(m_ref);
+    getfem::pintegration_method ppi = 
+      getfem::int_method_descriptor(INTEGRATION);
+    mim_ref.set_integration_method(m_ref.convex_index(), ppi);
+    
+    cout << "To ref L2 ERROR:"
+	 << getfem::asm_L2_dist(mim_ref, mf_ref, U, mf_ref, Uref) << endl;
+    
+    cout << "To ref H1 ERROR:"
+	 << getfem::asm_H1_dist(mim_ref, mf_ref, U, mf_ref, Uref) << endl;
+    
   }
 }
   
