@@ -17,17 +17,17 @@
 // along  with  this program;  if not, write to the Free Software Foundation,
 // Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// As a special exception, you may use this file as part of a free software
-// library without restriction.  Specifically, if other files instantiate
-// templates or use macros or inline functions from this file, or you compile
-// this file and link it with other files to produce an executable, this
-// file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however
-// invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.
+// As a special exception, you  may use  this file  as it is a part of a free
+// software  library  without  restriction.  Specifically,  if   other  files
+// instantiate  templates  or  use macros or inline functions from this file,
+// or  you compile this  file  and  link  it  with other files  to produce an
+// executable, this file  does  not  by itself cause the resulting executable
+// to be covered  by the GNU Lesser General Public License.  This   exception
+// does not  however  invalidate  any  other  reasons why the executable file
+// might be covered by the GNU Lesser General Public License.
 //
 //===========================================================================
-
+// $Id: getfemint_workspace.h 3224 2009-10-13 21:57:11Z lsaavedr $
 #ifndef GETFEMINT_WORKSPACE_H__
 #define GETFEMINT_WORKSPACE_H__
 
@@ -84,6 +84,7 @@ namespace getfemint
     void set_dependance(getfem_object *user, getfem_object *used_by);
     void sup_dependance(getfem_object *user, getfem_object *used_by);
 
+    void undelete_object(id_type id);
     /* at least mark the objet for future deletion (object becomes anonymous)
        and if possible, destroy the object (and all the objects which use this one
        if they are all anonymous) */
