@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //===========================================================================
 //
-// Copyright (C) 2000-2008 Yves Renard
+// Copyright (C) 2000-2010 Yves Renard
 //
 // This file is a part of GETFEM++
 //
@@ -62,7 +62,8 @@ namespace getfem {
     assem.push_vec(w);
     assem.assembly(rg);
     v.push_back(w[0]);
-    MPI_SUM_VECTOR(v);
+    w.resize(2);
+    MPI_SUM_VECTOR(v, w);
     return v[1]/v[0];
   }
 
@@ -347,9 +348,10 @@ namespace getfem {
   template <typename MAT, typename VECT>
   void asm_real_or_complex_1_param
   (MAT &M, const mesh_im &mim, const mesh_fem &mf_u, const mesh_fem &mf_data,
-   const VECT &A, const mesh_region &rg, const char *assembly_description) {
+   const VECT &A, const mesh_region &rg, const char *assembly_description,
+   const mesh_fem *mf_mult = 0) {
     asm_real_or_complex_1_param_
-      (M, mim, mf_u, mf_data, A, rg, assembly_description,
+      (M, mim, mf_u, mf_data, A, rg, assembly_description, mf_mult,
        typename gmm::linalg_traits<VECT>::value_type());
   }
 
@@ -358,11 +360,12 @@ namespace getfem {
   void asm_real_or_complex_1_param_
   (const MAT &M, const mesh_im &mim,  const mesh_fem &mf_u,
    const mesh_fem &mf_data, const VECT &A,  const mesh_region &rg,
-   const char *assembly_description, T) {
+   const char *assembly_description, const mesh_fem *mf_mult, T) {
     generic_assembly assem(assembly_description);
     assem.push_mi(mim);
     assem.push_mf(mf_u);
     assem.push_mf(mf_data);
+    if (mf_mult) assem.push_mf(*mf_mult);
     assem.push_data(A);
     assem.push_mat_or_vec(const_cast<MAT&>(M));
     assem.assembly(rg);
@@ -373,13 +376,13 @@ namespace getfem {
   void asm_real_or_complex_1_param_
   (MAT &M, const mesh_im &mim, const mesh_fem &mf_u, const mesh_fem &mf_data,
    const VECT &A, const mesh_region &rg,const char *assembly_description,
-   std::complex<T>) {
+   const mesh_fem *mf_mult, std::complex<T>) {
     asm_real_or_complex_1_param_(gmm::real_part(M),mim,mf_u,mf_data,
 				 gmm::real_part(A),rg,
-				 assembly_description, T());
+				 assembly_description, mf_mult, T());
     asm_real_or_complex_1_param_(gmm::imag_part(M),mim,mf_u,mf_data,
 				 gmm::imag_part(A),rg,
-				 assembly_description, T());
+				 assembly_description, mf_mult, T());
   }
 
   /** 
@@ -472,40 +475,89 @@ namespace getfem {
   void asm_source_term(const VECT1 &B, const mesh_im &mim, const mesh_fem &mf,
 		       const mesh_fem &mf_data, const VECT2 &F,
 		       const mesh_region &rg = mesh_region::all_convexes()) {
-    GMM_ASSERT1(mf_data.get_qdim() == 1,
-		"invalid data mesh fem (Qdim=1 required)");
+    GMM_ASSERT1(mf_data.get_qdim() == 1 ||
+		mf_data.get_qdim() == mf.get_qdim(),
+		"invalid data mesh fem (same Qdim or Qdim=1 required)");
 
     const char *st;
     if (mf.get_qdim() == 1)
       st = "F=data(#2); V(#1)+=comp(Base(#1).Base(#2))(:,j).F(j);";
-    else
+    else if (mf_data.get_qdim() == 1)
       st = "F=data(qdim(#1),#2);"
 	"V(#1)+=comp(vBase(#1).Base(#2))(:,i,j).F(i,j);";
+    else
+      st = "F=data(#2);"
+	"V(#1)+=comp(vBase(#1).vBase(#2))(:,i,j,i).F(j);";
     
     asm_real_or_complex_1_param(const_cast<VECT1 &>(B),mim,mf,mf_data,F,rg,st);
   }
 
   /** 
-      normal source term (for boundary (Neumann) condition).
+      source term (for both volumic sources and boundary (Neumann) sources).
+      For an homogeneous term.
       @ingroup asm
    */
   template<typename VECT1, typename VECT2>
-  void asm_normal_source_term(VECT1 &B, const mesh_im &mim, const mesh_fem &mf,
+  void asm_homogeneous_source_term(const VECT1 &B, const mesh_im &mim,
+				   const mesh_fem &mf, const VECT2 &F,
+		       const mesh_region &rg = mesh_region::all_convexes()) {
+    const char *st;
+    if (mf.get_qdim() == 1)
+      st = "F=data(1); V(#1)+=comp(Base(#1))(:).F(i);";
+    else
+      st = "F=data(qdim(#1)); V(#1)+=comp(vBase(#1))(:,i).F(i);";
+    
+    asm_real_or_complex_1_param(const_cast<VECT1 &>(B),mim,mf,mf,F,rg,st);
+  }
+
+
+  /** 
+      Normal source term (for boundary (Neumann) condition).
+      @ingroup asm
+   */
+  template<typename VECT1, typename VECT2>
+  void asm_normal_source_term(VECT1 &B, const mesh_im &mim,
+			      const mesh_fem &mf,
 			      const mesh_fem &mf_data, const VECT2 &F,
 			      const mesh_region &rg) {
-    GMM_ASSERT1(mf_data.get_qdim() == 1, "invalid data mesh_fem");
+    GMM_ASSERT1(mf_data.get_qdim() == 1 ||
+		mf_data.get_qdim() == mf.get_qdim(),
+		"invalid data mesh_fem (same Qdim or Qdim=1 required)");
 
     const char *st;
     if (mf.get_qdim() == 1)
       st = "F=data(mdim(#1),#2);"
 	"V(#1)+=comp(Base(#1).Base(#2).Normal())(:,j,k).F(k,j);";
-    else
+    else if (mf_data.get_qdim() == 1)
       st = "F=data(qdim(#1),mdim(#1),#2);"
 	"V(#1)+=comp(vBase(#1).Base(#2).Normal())(:,i,j,k).F(i,k,j);";
+    else
+      st = "F=data(mdim(#1),#2);"
+	"V(#1)+=comp(vBase(#1).vBase(#2).Normal())(:,i,j,i,k).F(k,j);";
 
     asm_real_or_complex_1_param(B, mim, mf, mf_data, F, rg, st);
   }
 
+  /** 
+      Homogeneous normal source term (for boundary (Neumann) condition).
+      @ingroup asm
+   */
+  template<typename VECT1, typename VECT2>
+  void asm_homogeneous_normal_source_term(VECT1 &B, const mesh_im &mim,
+					  const mesh_fem &mf,
+					  const VECT2 &F,
+					  const mesh_region &rg) {
+    const char *st;
+    if (mf.get_qdim() == 1)
+      st = "F=data(mdim(#1));"
+	"V(#1)+=comp(Base(#1).Normal())(:,k).F(k);";
+    else
+      st = "F=data(qdim(#1),mdim(#1));"
+	"V(#1)+=comp(vBase(#1).Normal())(:,i,j).F(i,j);";
+
+    asm_real_or_complex_1_param(B, mim, mf, mf, F, rg, st);
+  }
+
   template <typename V> bool is_Q_symmetric(const V& Q, size_type q,
 					    size_type nbd) {
     /* detect the symmetricity of Q (in that case the symmetricity of
@@ -545,8 +597,6 @@ namespace getfem {
     generic_assembly assem;
     GMM_ASSERT1(mf_d.get_qdim() == 1,
 		"invalid data mesh fem (Qdim=1 required)");
-    assert(mf_u.nb_dof() <= gmm::mat_nrows(M));
-    assert(mf_u.nb_dof() <= gmm::mat_ncols(M));
     const char *asm_str = "";
     if (mf_u.get_qdim() == 1)
       asm_str = "Q=data$1(#2);"
@@ -563,6 +613,27 @@ namespace getfem {
     asm_real_or_complex_1_param(M, mim, mf_u, mf_d, Q, rg, asm_str);
   }
 
+  template<typename MAT, typename VECT>
+  void asm_homogeneous_qu_term(MAT &M, const mesh_im &mim,
+			       const mesh_fem &mf_u, const VECT &Q, 
+			       const mesh_region &rg) {
+    generic_assembly assem;
+    const char *asm_str = "";
+    if (mf_u.get_qdim() == 1)
+      asm_str = "Q=data$1(1);"
+	"M(#1,#1)+=comp(Base(#1).Base(#1))(:,:).Q(i);";
+    else
+      if (is_Q_symmetric(Q,mf_u.get_qdim(),1))
+	asm_str = "Q=data$1(qdim(#1),qdim(#1));"
+		  "M(#1,#1)+=sym(comp(vBase(#1).vBase(#1))"
+		  "(:,i,:,j).Q(i,j));";
+      else
+        asm_str = "Q=data$1(qdim(#1),qdim(#1));"
+		  "M(#1,#1)+=comp(vBase(#1).vBase(#1))"
+		  "(:,i,:,j).Q(i,j);";
+    asm_real_or_complex_1_param(M, mim, mf_u, mf_u, Q, rg, asm_str);
+  }
+
   /** 
       Stiffness matrix for linear elasticity, with Lam coefficients
       @ingroup asm
@@ -600,6 +671,32 @@ namespace getfem {
     assem.assembly(rg);
   }
 
+
+  /** 
+      Stiffness matrix for linear elasticity, with constant Lam coefficients
+      @ingroup asm
+  */
+  template<class MAT, class VECT>
+  void asm_stiffness_matrix_for_homogeneous_linear_elasticity
+  (const MAT &RM_, const mesh_im &mim, const mesh_fem &mf,
+   const VECT &LAMBDA, const VECT &MU,
+   const mesh_region &rg = mesh_region::all_convexes()) {
+    MAT &RM = const_cast<MAT &>(RM_);
+    GMM_ASSERT1(mf.get_qdim() == mf.linked_mesh().dim(),
+		"wrong qdim for the mesh_fem");
+    generic_assembly assem("lambda=data$1(1); mu=data$2(1);"
+			   "t=comp(vGrad(#1).vGrad(#1));"
+                           "M(#1,#1)+= sym(t(:,i,j,:,i,j).mu(1)"
+			   "+ t(:,j,i,:,i,j).mu(1)"
+			   "+ t(:,i,i,:,j,j).lambda(1))");
+    assem.push_mi(mim);
+    assem.push_mf(mf);
+    assem.push_data(LAMBDA);
+    assem.push_data(MU);
+    assem.push_mat(RM);
+    assem.assembly(rg);
+  }
+
   /** 
       Stiffness matrix for linear elasticity, with a general Hooke
       tensor. This is more a demonstration of generic assembly than
@@ -775,6 +872,20 @@ namespace getfem {
 				"(:,i,:,j,k).a(j,i,k)");
   }
 
+  /** The same but with a constant matrix
+   */
+  template<typename MAT, typename VECT>
+  void asm_stiffness_matrix_for_homogeneous_scalar_elliptic
+  (MAT &M, const mesh_im &mim, const mesh_fem &mf,
+   const VECT &A, const mesh_region &rg = mesh_region::all_convexes()) {
+    /*GMM_ASSERT1(mf_data.get_qdim() == 1,
+      "invalid data mesh fem (Qdim=1 required)");*/
+    asm_real_or_complex_1_param(M,mim,mf,mf,A,rg,
+				"a=data$1(mdim(#1),mdim(#1));"
+				"M$1(#1,#1)+=comp(Grad(#1).Grad(#1))"
+				"(:,i,:,j).a(j,i)");
+  }
+
   /** The same but on each component of mf when mf has a qdim > 1 
    */
   template<typename MAT, typename VECT>
@@ -790,9 +901,24 @@ namespace getfem {
        "(:,l,i,:,l,j,k).a(j,i,k)");
   }
 
+  /** The same but with a constant matrix 
+   */
+  template<typename MAT, typename VECT>
+  void asm_stiffness_matrix_for_homogeneous_scalar_elliptic_componentwise
+  (MAT &M, const mesh_im &mim, const mesh_fem &mf, const VECT &A, 
+   const mesh_region &rg = mesh_region::all_convexes()) {
+    /* GMM_ASSERT1(mf_data.get_qdim() == 1,
+       "invalid data mesh fem (Qdim=1 required)");*/
+    asm_real_or_complex_1_param
+      (M,mim,mf,mf,A,rg, "a=data$1(mdim(#1),mdim(#1));"
+       "M$1(#1,#1)+=comp(vGrad(#1).vGrad(#1))"
+       "(:,l,i,:,l,j).a(j,i)");
+  }
+
+
   /**
      Assembly of @f$\int_\Omega A(x)\nabla u.\nabla v@f$, where @f$A(x)@f$
-     is a NxNxNxN (symmetric positive definite) tensor.
+     is a NxNxNxN (symmetric positive definite) tensor defined on mf_data.
   */
   template<typename MAT, typename VECT> void
   asm_stiffness_matrix_for_vector_elliptic
@@ -805,15 +931,33 @@ namespace getfem {
     */
     asm_real_or_complex_1_param
       (M,mim,mf,mf_data,A,rg, 
-       "a=data$1(mdim(#1),mdim(#1),mdim(#1),mdim(#1),#2);"
+       "a=data$1(qdim(#1),mdim(#1),qdim(#1),mdim(#1),#2);"
        "t=comp(vGrad(#1).vGrad(#1).Base(#2));"
-       "M(#1,#1)+= sym(t(:,i,j,:,k,l,p).a(i,j,k,l,p))");
+       "M(#1,#1)+= t(:,i,j,:,k,l,p).a(i,j,k,l,p)");
+  }
+
+  /**
+     Assembly of @f$\int_\Omega A(x)\nabla u.\nabla v@f$, where @f$A(x)@f$
+     is a NxNxNxN (symmetric positive definite) constant tensor.
+  */
+  template<typename MAT, typename VECT> void
+  asm_stiffness_matrix_for_homogeneous_vector_elliptic
+  (MAT &M, const mesh_im &mim, const mesh_fem &mf,
+   const VECT &A, const mesh_region &rg = mesh_region::all_convexes()) {
+    /* 
+       M = a_{i,j,k,l}D_{i,j}(u)D_{k,l}(v)
+    */
+    asm_real_or_complex_1_param
+      (M,mim,mf,mf,A,rg, 
+       "a=data$1(qdim(#1),mdim(#1),qdim(#1),mdim(#1));"
+       "t=comp(vGrad(#1).vGrad(#1));"
+       "M(#1,#1)+= t(:,i,j,:,k,l).a(i,j,k,l)");
   }
 
 
 
   /** 
-      assembly of the term @f$\int_\Omega Ku.v - \nabla u.\nabla v@f$, 
+      assembly of the term @f$\int_\Omega Kuv - \nabla u.\nabla v@f$, 
       for the helmholtz equation (@f$\Delta u + k^2u = 0@f$, with @f$K=k^2@f$).
 
       The argument K_squared may be a real or a complex-valued vector.
@@ -882,6 +1026,80 @@ namespace getfem {
     assem.assembly(rg);
   }
 
+
+  /** 
+      assembly of the term @f$\int_\Omega Kuv - \nabla u.\nabla v@f$, 
+      for the helmholtz equation (@f$\Delta u + k^2u = 0@f$, with @f$K=k^2@f$).
+
+      The argument K_squared may be a real or a complex-valued scalar.
+
+     @ingroup asm
+  */
+  template<typename MAT, typename VECT>
+  void asm_homogeneous_Helmholtz
+  (MAT &M, const mesh_im &mim, const mesh_fem &mf_u, const VECT &K_squared, 
+   const mesh_region &rg = mesh_region::all_convexes()) {
+    asm_homogeneous_Helmholtz(M, mim, mf_u, K_squared, rg,
+		  typename gmm::linalg_traits<VECT>::value_type());
+  }
+
+  template<typename MAT, typename VECT, typename T>
+  void asm_homogeneous_Helmholtz(MAT &M, const mesh_im &mim,
+				 const mesh_fem &mf_u,
+				 const VECT &K_squared,
+				 const mesh_region &rg, T) {
+    asm_homogeneous_Helmholtz_real(M, mim, mf_u, K_squared, rg);
+  }
+
+  template<typename MAT, typename VECT, typename T>
+  void asm_homogeneous_Helmholtz(MAT &M, const mesh_im &mim,
+				 const mesh_fem &mf_u,
+				 const VECT &K_squared,
+				 const mesh_region &rg, std::complex<T>) {
+    asm_homogeneous_Helmholtz_cplx(gmm::real_part(M),
+				   gmm::imag_part(M), mim, mf_u,
+				   gmm::real_part(K_squared),
+				   gmm::imag_part(K_squared), rg);
+  }
+
+
+  template<typename MATr, typename MATi, typename VECTr, typename VECTi>  
+  void asm_homogeneous_Helmholtz_cplx(const MATr &Mr, const MATi &Mi,
+				      const mesh_im &mim,
+				      const mesh_fem &mf_u,
+				      const VECTr &K_squaredr,
+				      const VECTi &K_squaredi, 
+				      const mesh_region &rg) {
+    generic_assembly assem("Kr=data$1(1); Ki=data$2(1);"
+			   "m = comp(Base(#1).Base(#1)); "
+			   "M$1(#1,#1)+=sym(m(:,:).Kr(j) - "
+			   "comp(Grad(#1).Grad(#1))(:,i,:,i));"
+			   "M$2(#1,#1)+=sym(m(:,:).Ki(j));");
+    assem.push_mi(mim);
+    assem.push_mf(mf_u);
+    assem.push_data(K_squaredr);
+    assem.push_data(K_squaredi);
+    assem.push_mat(const_cast<MATr&>(Mr));
+    assem.push_mat(const_cast<MATi&>(Mi));
+    assem.assembly(rg);
+  }
+
+  template<typename MAT, typename VECT>  
+  void asm_homogeneous_Helmholtz_real(const MAT &M, const mesh_im &mim,
+				      const mesh_fem &mf_u,
+				      const VECT &K_squared, 
+				      const mesh_region &rg) {
+    generic_assembly assem("K=data(1);"
+			   "m = comp(Base(#1).Base(#1)); "
+			   "M$1(#1,#1)+=sym(m(:,:).K(j) - "
+			   "comp(Grad(#1).Grad(#1))(:,i,:,i));");
+    assem.push_mi(mim);
+    assem.push_mf(mf_u);
+    assem.push_data(K_squared);
+    assem.push_mat(const_cast<MAT&>(M));
+    assem.assembly(rg);
+  }
+
   enum { ASMDIR_BUILDH = 1, ASMDIR_BUILDR = 2, ASMDIR_SIMPLIFY = 4,
 	 ASMDIR_BUILDALL = 7 };
 
@@ -912,25 +1130,32 @@ namespace getfem {
     typedef typename gmm::linalg_traits<VECT1>::value_type value_type;
     typedef typename gmm::number_traits<value_type>::magnitude_type magn_type;
 
+    if ((version & ASMDIR_SIMPLIFY) &&
+	(mf_u.is_reduced() || mf_mult.is_reduced() || mf_r.is_reduced())) {
+      GMM_WARNING1("Sorry, no simplification for reduced fems");
+      version = (version & (ASMDIR_BUILDR | ASMDIR_BUILDH));
+    }      
     
     region.from_mesh(mim.linked_mesh()).error_if_not_faces();
     GMM_ASSERT1(mf_r.get_qdim() == 1,
 		"invalid data mesh fem (Qdim=1 required)");
     if (version & ASMDIR_BUILDH) {
       asm_mass_matrix(H, mim, mf_mult, mf_u, region);
-      gmm::clean(H, gmm::default_tol(magn_type())
-		 * gmm::mat_maxnorm(H) * magn_type(1000));
+//       gmm::clean(H, gmm::default_tol(magn_type()) //  remettre !
+// 		 * gmm::mat_maxnorm(H) * magn_type(1000));
     }
     if (version & ASMDIR_BUILDR)
       asm_source_term(R, mim, mf_mult, mf_r, r_data, region);
 
+    return; //  enlever !
+
     // Verifications and simplifications
 
     pfem pf_u, pf_r, pf_m;
     bool warning_msg1 = false, warning_msg2 = false;
     dal::bit_vector simplifiable_dofs, nonsimplifiable_dofs;
-    std::vector<size_type> simplifiable_indices(mf_mult.nb_dof());
-    std::vector<value_type> simplifiable_values(mf_mult.nb_dof());
+    std::vector<size_type> simplifiable_indices(mf_mult.nb_basic_dof());
+    std::vector<value_type> simplifiable_values(mf_mult.nb_basic_dof());
     std::vector<value_type> v1, v2, v3;
 
     for (mr_visitor v(region); !v.finished(); v.next()) {
@@ -956,11 +1181,11 @@ namespace getfem {
       if (!(version & ASMDIR_SIMPLIFY)) continue;
       
       mesh_fem::ind_dof_face_ct pf_u_ct
-	= mf_u.ind_dof_of_face_of_element(cv, f);
+	= mf_u.ind_basic_dof_of_face_of_element(cv, f);
       mesh_fem::ind_dof_face_ct pf_r_ct
-	= mf_r.ind_dof_of_face_of_element(cv, f);
+	= mf_r.ind_basic_dof_of_face_of_element(cv, f);
       mesh_fem::ind_dof_face_ct pf_m_ct
-	= mf_mult.ind_dof_of_face_of_element(cv, f);
+	= mf_mult.ind_basic_dof_of_face_of_element(cv, f);
       
       size_type pf_u_nbdf = pf_u_ct.size();
       size_type pf_m_nbdf = pf_m_ct.size();
@@ -995,59 +1220,6 @@ namespace getfem {
 	    = r_data[pf_r_ct[i/Qratio]*Qratio+(i%Qratio)];
 	}
       }
-//       else { // local inversion of the mass matrix.
-// 	cout << "local inv\n";
-// 	bgeot::base_tensor t1, t2;
-// 	pintegration_method pim = mim.int_method_of_element(cv);
-// 	bgeot::pgeometric_trans pgt = mf_u.linked_mesh().trans_of_convex(cv);
-// 	getfem::pmat_elem_type pme1 =
-// 	  getfem::mat_elem_product(getfem::mat_elem_base(pf_m),
-// 				   getfem::mat_elem_base(pf_u));
-// 	getfem::mat_elem(pme1,pim,pgt)->gen_compute_on_face
-// 	  (t1, mf_u.linked_mesh().points_of_convex(cv), f, cv);
-// 	getfem::pmat_elem_type pme2 =
-// 	  getfem::mat_elem_product(getfem::mat_elem_base(pf_m),
-// 				   getfem::mat_elem_base(pf_r));
-// 	getfem::mat_elem(pme2, pim, pgt)->gen_compute_on_face
-// 	  (t2, mf_u.linked_mesh().points_of_convex(cv), f, cv);
-	
-// 	base_matrix m1(pf_m_nbdf_loc, pf_u_nbdf_loc);
-// 	base_matrix m2(pf_m_nbdf_loc, pf_r_nbdf_loc);
-	
-// 	for (size_type i = 0; i < pf_m_nbdf_loc; ++i)
-// 	  for (size_type j = 0; j < pf_u_nbdf_loc; ++j)
-// 	    m1(i, j) = t1(pf_m->structure(cv)->ind_points_of_face(f)[i],
-// 			  pf_u->structure(cv)->ind_points_of_face(f)[j]);
-// 	for (size_type i = 0; i < pf_m_nbdf_loc; ++i)
-// 	  for (size_type j = 0; j < pf_r_nbdf_loc; ++j)
-// 	    m2(i, j) = t2(pf_m->structure(cv)->ind_points_of_face(f)[i],
-// 			  pf_r->structure(cv)->ind_points_of_face(f)[j]);
-	
-// 	gmm::lu_inverse(m1);
-// 	size_type Q =  pf_u_nbdf / pf_u_nbdf_loc;
-// 	size_type Qu = mf_u.get_qdim();
-	
-// 	v1.resize(pf_r_nbdf_loc * (Qu / Q));
-// 	v2.resize(pf_m_nbdf_loc);
-// 	v3.resize(pf_u_nbdf_loc);
-
-// 	for (size_type k = 0; k < Q; ++k) {
-// 	  if (Q > 1 || (Qu == 1)) {
-// 	    for (size_type i = 0; i < pf_r_nbdf_loc; ++i)
-// 	      v1[i] = r_data[pf_r_ct[i]*Q+k];
-// 	  }
-// 	  else {
-// 	    for (size_type l = 0; l < Qu; ++l)
-// 	      for (size_type i = 0; i < pf_r_nbdf_loc; ++i)
-// 		v1[i*Qu+l] = r_data[pf_r_ct[i]*Qu+l];
-// 	  }
-// 	  gmm::mult(m2, v1, v2);
-// 	  gmm::mult(m1, v2, v3);
-	  
-// 	  for (size_type i = 0; i < pf_m_nbdf_loc; ++i)
-// 	    simplifiable_values[pf_m_ct[i*Q+k]] = v3[i];
-// 	}
-//       }
     }
     
     if (version & ASMDIR_SIMPLIFY) {
@@ -1061,11 +1233,11 @@ namespace getfem {
       for (dal::bv_visitor i(simplifiable_dofs); !i.finished(); ++i)
 	if (!(nonsimplifiable_dofs[i])) {
 	  if (version & ASMDIR_BUILDH) {  /* "erase" the row i */
-	    const mesh::ind_cv_ct &cv_ct = mf_mult.convex_to_dof(i);
+	    const mesh::ind_cv_ct &cv_ct = mf_mult.convex_to_basic_dof(i);
 	    for (size_type j = 0; j < cv_ct.size(); ++j) {
 	      size_type cv = cv_ct[j];
-	      for (size_type k=0; k < mf_u.nb_dof_of_element(cv); ++k)
-		H(i, mf_u.ind_dof_of_element(cv)[k]) = value_type(0);
+	      for (size_type k=0; k < mf_u.nb_basic_dof_of_element(cv); ++k)
+		H(i, mf_u.ind_basic_dof_of_element(cv)[k]) = value_type(0);
 	    }
 	    H(i, simplifiable_indices[i]) = value_type(1);
 	  }
@@ -1136,12 +1308,12 @@ namespace getfem {
   /**
      Assembly of generalized Dirichlet constraints h(x)u(x) = r(x),
      where h is a QxQ matrix field (Q == mf_u.get_qdim()), outputs a
-     (under-determined) linear system MU=B.
+     (under-determined) linear system HU=R.
 
      size(h_data) = Q^2 * nb_dof(mf_rh);
      size(r_data) = Q   * nb_dof(mf_rh);
 
-     This function tries hard to make M diagonal or mostly diagonal:
+     This function tries hard to make H diagonal or mostly diagonal:
      this function is able to "simplify" the dirichlet constraints (see below)
      version = |ASMDIR_BUILDH : build H
      |ASMDIR_BUILDR : build R
@@ -1159,13 +1331,19 @@ namespace getfem {
    int version =  ASMDIR_BUILDALL) {
     pfem pf_u, pf_rh;
 
+    if ((version & ASMDIR_SIMPLIFY) &&
+	(mf_u.is_reduced() || mf_h.is_reduced() || mf_r.is_reduced())) {
+      GMM_WARNING1("Sorry, no simplification for reduced fems");
+      version = (version & ASMDIR_BUILDR);
+    }
+
     region.from_mesh(mim.linked_mesh()).error_if_not_faces();
     GMM_ASSERT1(mf_h.get_qdim() == 1 && mf_r.get_qdim() == 1,
 		"invalid data mesh fem (Qdim=1 required)");
     if (version & ASMDIR_BUILDH) {
       asm_qu_term(H, mim, mf_u, mf_h, h_data, region);
       std::vector<size_type> ind(0);
-      dal::bit_vector bdof = mf_u.dof_on_set(region);
+      dal::bit_vector bdof = mf_u.basic_dof_on_region(region);
       // gmm::clean(H, 1E-15 * gmm::mat_maxnorm(H));
       for (size_type i = 0; i < mf_u.nb_dof(); ++i)
 	if (!(bdof[i])) ind.push_back(i);
@@ -1220,19 +1398,19 @@ namespace getfem {
 		< 1.0E-14) {
 	      /* the dof might be "duplicated" */
 	      for (size_type q = 0; q < Q; ++q) {
-		size_type dof_u = mf_u.ind_dof_of_element(cv)[ind_u*Q + q];
+		size_type dof_u = mf_u.ind_basic_dof_of_element(cv)[ind_u*Q + q];
 		
 		/* "erase" the row */
 		if (version & ASMDIR_BUILDH)
-		  for (size_type k=0; k < mf_u.nb_dof_of_element(cv); ++k)
-		    H(dof_u, mf_u.ind_dof_of_element(cv)[k]) = 0.0;
+		  for (size_type k=0; k < mf_u.nb_basic_dof_of_element(cv); ++k)
+		    H(dof_u, mf_u.ind_basic_dof_of_element(cv)[k]) = 0.0;
 		
-		size_type dof_rh = mf_r.ind_dof_of_element(cv)[ind_rh];
+		size_type dof_rh = mf_r.ind_basic_dof_of_element(cv)[ind_rh];
 		/* set the "simplified" row */
 		if (version & ASMDIR_BUILDH)
 		  for (unsigned jj=0; jj < Q; jj++) {
 		    size_type dof_u2
-		      = mf_u.ind_dof_of_element(cv)[ind_u*Q+jj];
+		      = mf_u.ind_basic_dof_of_element(cv)[ind_u*Q+jj];
 		    H(dof_u, dof_u2) = h_data[(jj*Q+q) + Q*Q*(dof_rh)];
 		  }
 		if (version & ASMDIR_BUILDR) R[dof_u] = r_data[dof_rh*Q+q];
@@ -1262,38 +1440,36 @@ namespace getfem {
    const VECT2 &F) {
     // Marche uniquement pour des ddl de lagrange.
     size_type Q=mf.get_qdim();
-    dal::bit_vector nndof = mf.dof_on_set(boundary);
+    GMM_ASSERT1(!(mf.is_reduced()), "This function is not adapted to "
+		"reduced finite element methods"); 
+    dal::bit_vector nndof = mf.basic_dof_on_region(boundary);
     pfem pf1;
     for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
       pf1 = mf.fem_of_element(cv);
       pdof_description ldof = lagrange_dof(pf1->dim());
       size_type nbd = pf1->nb_dof(cv);
-      for (size_type i = 0; i < nbd; i++)
-	{
-	  size_type dof1 = mf.ind_dof_of_element(cv)[i*Q];
-	  if (nndof.is_in(dof1) && pf1->dof_types()[i] == ldof)
-	    {
-	      // cout << "dof : " << i << endl;
-	      for (size_type j = 0; j < nbd; j++)
-		{
-		  size_type dof2 = mf.ind_dof_of_element(cv)[j*Q];
-		  for (size_type k = 0; k < Q; ++k)
-		    for (size_type l = 0; l < Q; ++l)
-		      {
-			if (!(nndof.is_in(dof2)) &&
-			    dof_compatibility(pf1->dof_types()[j],
-					      lagrange_dof(pf1->dim())))
-			  B[dof2+k] -= RM(dof2+k, dof1+l) * F[dof1+l];
-			RM(dof2+k, dof1+l) = RM(dof1+l, dof2+k) = 0;
-		      }
-		}
-	      for (size_type k = 0; k < Q; ++k)
-		{ RM(dof1+k, dof1+k) = 1; B[dof1+k] = F[dof1+k]; }
-	    }
+      for (size_type i = 0; i < nbd; i++) {
+	size_type dof1 = mf.ind_basic_dof_of_element(cv)[i*Q];
+	if (nndof.is_in(dof1) && pf1->dof_types()[i] == ldof) {
+	  // cout << "dof : " << i << endl;
+	  for (size_type j = 0; j < nbd; j++) {
+	    size_type dof2 = mf.ind_basic_dof_of_element(cv)[j*Q];
+	    for (size_type k = 0; k < Q; ++k)
+	      for (size_type l = 0; l < Q; ++l) {
+		if (!(nndof.is_in(dof2)) &&
+		    dof_compatibility(pf1->dof_types()[j],
+				      lagrange_dof(pf1->dim())))
+		  B[dof2+k] -= RM(dof2+k, dof1+l) * F[dof1+l];
+		RM(dof2+k, dof1+l) = RM(dof1+l, dof2+k) = 0;
+	      }
+	  }
+	  for (size_type k = 0; k < Q; ++k)
+	    { RM(dof1+k, dof1+k) = 1; B[dof1+k] = F[dof1+k]; }
 	}
+      }
     }
   }
-
+  
   /* add a dirichlet condition on a single dof, modifiying the matrix
      RM and the rhs B.  (keeping the symmetry properties) */
   template<typename MATRM, typename VECT1>
