@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //===========================================================================
 //
-// Copyright (C) 2004-2008 Yves Renard
+// Copyright (C) 2004-2009 Yves Renard
 //
 // This file is a part of GETFEM++
 //
@@ -17,14 +17,14 @@
 // along  with  this program;  if not, write to the Free Software Foundation,
 // Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// As a special exception, you may use this file as part of a free software
-// library without restriction.  Specifically, if other files instantiate
-// templates or use macros or inline functions from this file, or you compile
-// this file and link it with other files to produce an executable, this
-// file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however
-// invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.
+// As a special exception, you  may use  this file  as it is a part of a free
+// software  library  without  restriction.  Specifically,  if   other  files
+// instantiate  templates  or  use macros or inline functions from this file,
+// or  you compile this  file  and  link  it  with other files  to produce an
+// executable, this file  does  not  by itself cause the resulting executable
+// to be covered  by the GNU Lesser General Public License.  This   exception
+// does not  however  invalidate  any  other  reasons why the executable file
+// might be covered by the GNU Lesser General Public License.
 //
 //===========================================================================
 
@@ -81,9 +81,8 @@ namespace getfem {
 		      std::vector<bool> &ids) const;
   public:
     template <typename IT_LS_ENRICH>
-    fem_level_set(IT_LS_ENRICH it,pfem pf, const mesh_level_set &mls_,
-		  size_type xfi) : 
-      bfem(pf), mls(mls_), xfem_index(xfi) {
+    fem_level_set(IT_LS_ENRICH it, pfem pf, const mesh_level_set &mls_,
+		  size_type xfi) : bfem(pf), mls(mls_), xfem_index(xfi) {
       dofzones.assign(it, it + bfem->nb_dof(0));
       init();
     }
