@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //===========================================================================
 //
-// Copyright (C) 2001-2008 Yves Renard
+// Copyright (C) 2001-2010 Yves Renard
 //
 // This file is a part of GETFEM++
 //
@@ -68,12 +68,14 @@ namespace getfem {
     void points_on_convex(size_type i, std::vector<size_type> &itab) const;
     const std::vector<base_node> &reference_coords(void) { return ref_coords; }
 
-    /* extrapolation = false : Only the points inside the mesh are distributed.
-     * extrapolation = true  : Try to project the exterior points.
-     * TODO : for extrapolation, verify that all the points have been taken
-     *        into account, else test them on the frontiere convexes.
+    /* extrapolation = 1 : Only the points inside the mesh are distributed.
+     * extrapolation = 2 : Try to extrapolate the exterior points near the
+     *                     boundary.
+     * extrapolation = 3 : Extrapolate all the exterior points. Could be
+     *                     expensive.
+     * 
      */
-    void distribute(bool extrapolation = false);
+    void distribute(int extrapolation = 0);
     mesh_trans_inv(const mesh &m, double EPS_ = 1E-12)
       : bgeot::geotrans_inv(EPS_), msh(m) {}
   private :
@@ -96,7 +98,7 @@ namespace getfem {
     GMM_ASSERT1(gmm::vect_size(V) == mf.nb_dof() && Q == 1,
 		"Dof vector has not the right size");
     for (dal::bv_visitor i(dofs); !i.finished(); ++i)
-      V[i] = f(mf.point_of_dof(i));
+      V[i] = f(mf.point_of_basic_dof(i));
   }
 
   template <typename VECT, typename F, typename M>
@@ -108,7 +110,7 @@ namespace getfem {
 		"Dof vector has not the right size");
     for (dal::bv_visitor i(dofs); !i.finished(); ++i)
       if (i % Q == 0)
-	gmm::copy(f(mf.point_of_dof(i)),
+	gmm::copy(f(mf.point_of_basic_dof(i)),
 		  gmm::sub_vector(V, gmm::sub_interval(i*N/Q, N)));
   }
 
@@ -116,6 +118,7 @@ namespace getfem {
   inline void interpolation_function__(const mesh_fem &mf, VECT &V,
 				       F &f, const dal::bit_vector &dofs,
 				       const M &mm, gmm::abstract_matrix) {
+    typedef typename gmm::linalg_traits<VECT>::value_type T;
     size_type Nr = gmm::mat_nrows(mm), Nc = gmm::mat_ncols(mm), N = Nr*Nc;
     size_type Q = mf.get_qdim();
     base_matrix m(Nr, Nc);
@@ -123,11 +126,12 @@ namespace getfem {
 		"Dof vector has not the right size");
     for (dal::bv_visitor i(dofs); !i.finished(); ++i)
       if (i % Q == 0) {
-	gmm::copy(f(mf.point_of_dof(i)), m);
+	gmm::copy(f(mf.point_of_basic_dof(i)), m);
 	for (size_type j = 0; j < Nc; ++j)
 	  gmm::copy(gmm::mat_col(m, j),
 		    gmm::sub_vector(V, gmm::sub_interval(i*N/Q+j*Nr, Nr)));
       }
+
   }
 
   template <typename VECT, typename F, typename M>
@@ -155,15 +159,6 @@ namespace getfem {
     }
     return op;
   }
-      
-  template <typename VECT> inline void MPI_MERGE_VECTOR(VECT& V) {
-    typedef typename gmm::linalg_traits<VECT>::value_type T;
-    std::vector<T> W(gmm::vect_size(V)); gmm::copy(V, W);
-    MPI_Allreduce(&(V[0]), &(W[0]), gmm::vect_size(V), mpi_type(T()),
-		  mpi_take_one_op<T>, MPI_COMM_WORLD);
-  }
-#else
-  template <typename VECT> inline void MPI_MERGE_VECTOR(VECT& ) {}
 #endif
 
   // TODO : verify that rhs is a lagrange fem
@@ -177,15 +172,29 @@ namespace getfem {
      resulting vector V is distributed.
   */
   template <typename VECT, typename F>
-  void interpolation_function(mesh_fem &mf_target, const VECT &V, F &f,
+  void interpolation_function(mesh_fem &mf_target, const VECT &VV, F &f,
 			 mesh_region rg=mesh_region::all_convexes()) {
-    gmm::clear(const_cast<VECT &>(V));
+    typedef typename gmm::linalg_traits<VECT>::value_type T;
+    size_type qqdimt = gmm::vect_size(VV) / mf_target.nb_dof();
+    std::vector<T> V(mf_target.nb_basic_dof()*qqdimt);
     mf_target.linked_mesh().intersect_with_mpi_region(rg);
-    dal::bit_vector dofs = mf_target.dof_on_set(rg);
+    dal::bit_vector dofs = mf_target.basic_dof_on_region(rg);
     if (dofs.card() > 0)
-      interpolation_function_(mf_target, const_cast<VECT &>(V), f, dofs,
-			      f(mf_target.point_of_dof(dofs.first())));
-    //MPI_MERGE_VECTOR(V);
+      interpolation_function_(mf_target, V, f, dofs,
+			      f(mf_target.point_of_basic_dof(dofs.first())));
+
+    if (mf_target.is_reduced()) {
+      for (size_type k = 0; k < qqdimt; ++k)
+	gmm::mult(mf_target.reduction_matrix(),
+		  gmm::sub_vector(V,
+				  gmm::sub_slice(k, mf_target.nb_basic_dof(),
+						 qqdimt)),
+		  gmm::sub_vector(const_cast<VECT &>(VV),
+				  gmm::sub_slice(k, mf_target.nb_dof(),
+						 qqdimt)));
+    }
+    else
+      gmm::copy(V, const_cast<VECT &>(VV));
   }
 
   /* ********************************************************************* */
@@ -209,7 +218,7 @@ namespace getfem {
   */
   template<typename VECTU, typename VECTV>
   void interpolation(const mesh_fem &mf_source, const mesh_fem &mf_target,
-		     const VECTU &U, VECTV &V, bool extrapolation = false,
+		     const VECTU &U, VECTV &V, int extrapolation = 0,
 		     double EPS = 1E-10);
 
   /**
@@ -221,7 +230,7 @@ namespace getfem {
    */
   template<typename MAT>
   void interpolation(const mesh_fem &mf_source, const mesh_fem &mf_target,
-		     MAT &M, bool extrapolation = false, double EPS = 1E-10);
+		     MAT &M, int extrapolation = 0, double EPS = 1E-10);
 
 
   /* --------------------------- Implementation ---------------------------*/
@@ -238,12 +247,13 @@ namespace getfem {
   template<typename VECTU, typename VECTV, typename MAT>
     void interpolation_same_mesh(const mesh_fem &mf_source,
 				 const mesh_fem &mf_target,
-				 const VECTU &U, VECTV &V,
-				 MAT &M, int version) {
+				 const VECTU &UU, VECTV &VV,
+				 MAT &MM, int version) {
+
     typedef typename gmm::linalg_traits<VECTU>::value_type T;
     base_matrix G;
     dim_type qdim = mf_source.get_qdim();
-    dim_type qqdim = dim_type(gmm::vect_size(U)/mf_source.nb_dof());
+    dim_type qqdim = dim_type(gmm::vect_size(UU)/mf_source.nb_dof());
     std::vector<T> val(qdim);
     std::vector<std::vector<T> > coeff;
     std::vector<size_type> dof_source;
@@ -252,11 +262,17 @@ namespace getfem {
 		<< qdim << " on a mesh_fem whose Qdim is " << 
 		int(mf_target.get_qdim()));
     size_type qmult = mf_source.get_qdim()/mf_target.get_qdim();
+    size_type qqdimt = qqdim * mf_source.get_qdim()/mf_target.get_qdim();
     fem_precomp_pool fppool;
     dal::bit_vector dof_t_done;
-    
-    dof_t_done.sup(0, mf_target.nb_dof());
-    gmm::clear(M);
+    std::vector<T> U(mf_source.nb_basic_dof()*qqdim);
+    std::vector<T> V(mf_target.nb_basic_dof()*qqdimt);
+    gmm::row_matrix<gmm::rsvector<scalar_type> >
+      M(mf_target.nb_basic_dof(), mf_source.nb_basic_dof());
+
+    if (version == 0) mf_source.extend_vector(UU, U);
+      
+    dof_t_done.sup(0, mf_target.nb_basic_dof());
 
     /* we should sort convexes by their fem */
     for (dal::bv_visitor cv(mf_source.convex_index()); !cv.finished(); ++cv) {
@@ -268,30 +284,31 @@ namespace getfem {
       size_type nbd_s = pf_s->nb_dof(cv);
       size_type nbd_t = pf_t->nb_dof(cv);
       mesh_fem::ind_dof_ct::const_iterator itdof;
-      size_type cvnbdof = mf_source.nb_dof_of_element(cv);
+      size_type cvnbdof = mf_source.nb_basic_dof_of_element(cv);
 
       if (version == 0) {
         coeff.resize(qqdim);
         for (size_type qq=0; qq < qqdim; ++qq) {
           coeff[qq].resize(nbd_s*qdim);
-          itdof = mf_source.ind_dof_of_element(cv).begin();
+          itdof = mf_source.ind_basic_dof_of_element(cv).begin();
           for (size_type k = 0; k < cvnbdof; ++k, ++itdof) {
 	    coeff[qq][k] = U[(*itdof)*qqdim+qq];
 	  }
         }
       }
       if (pf_s->need_G()) 
-	bgeot::vectors_to_base_matrix(G,
-			      mf_source.linked_mesh().points_of_convex(cv));
+	bgeot::vectors_to_base_matrix
+	  (G, mf_source.linked_mesh().points_of_convex(cv));
 
       GMM_ASSERT1(pf_t->target_dim() == 1,
 		  "won't interpolate on a vector FEM... ");
       pfem_precomp pfp = fppool(pf_s, pf_t->node_tab(cv));
       fem_interpolation_context ctx(pgt,pfp,size_type(-1), G, cv,
 				    size_type(-1));
-      itdof = mf_target.ind_dof_of_element(cv).begin();
+      itdof = mf_target.ind_basic_dof_of_element(cv).begin();
       if (version != 0) {
-	const mesh_fem::ind_dof_ct &idct = mf_source.ind_dof_of_element(cv);
+	const mesh_fem::ind_dof_ct &idct
+	  = mf_source.ind_basic_dof_of_element(cv);
 	dof_source.assign(idct.begin(), idct.end());
       }
       for (size_type i = 0; i < nbd_t; ++i, itdof+=mf_target.get_qdim()) {
@@ -307,7 +324,7 @@ namespace getfem {
           }
 	}
 	else {
-	  base_matrix Mloc(qdim, mf_source.nb_dof_of_element(cv));
+	  base_matrix Mloc(qdim, mf_source.nb_basic_dof_of_element(cv));
 	  pf_s->interpolation(ctx, Mloc, qdim);
 	  for (size_type k=0; k < qdim; ++k) {
             for (size_type j=0; j < dof_source.size(); ++j) {
@@ -317,6 +334,26 @@ namespace getfem {
 	}
       }
     }
+
+    
+    if (version == 0)
+      mf_target.reduce_vector(V, VV);
+    else {
+      if (mf_target.is_reduced())
+	if (mf_source.is_reduced()) {
+	  gmm::row_matrix<gmm::rsvector<scalar_type> >
+	    MMM(mf_target.nb_dof(), mf_source.nb_basic_dof());
+	  gmm::mult(mf_target.reduction_matrix(), M, MMM);
+	  gmm::mult(MMM, mf_source.extension_matrix(), MM);
+	}
+	else
+	  gmm::mult(mf_target.reduction_matrix(), M, MM); 
+      else
+	if (mf_source.is_reduced())
+	  gmm::mult(M, mf_source.extension_matrix(), MM);
+	else
+	  gmm::copy(M, MM);
+    }
   }
 
   
@@ -328,20 +365,25 @@ namespace getfem {
   template<typename VECTU, typename VECTV, typename MAT>
   void interpolation(const mesh_fem &mf_source,
 		     mesh_trans_inv &mti,
-		     const VECTU &U, VECTV &V, MAT &M,
-		     int version, bool extrapolation = false) {
+		     const VECTU &UU, VECTV &V, MAT &MM,
+		     int version, int extrapolation = 0,
+		     dal::bit_vector *dof_untouched = 0) {
 
     typedef typename gmm::linalg_traits<VECTU>::value_type T;
     const mesh &msh(mf_source.linked_mesh());
     dim_type qdim_s = mf_source.get_qdim();
-    dim_type qqdim = dim_type(gmm::vect_size(U)/mf_source.nb_dof());
+    dim_type qqdim = dim_type(gmm::vect_size(UU)/mf_source.nb_dof());
+
+    std::vector<T> U(mf_source.nb_basic_dof()*qqdim);
+    gmm::row_matrix<gmm::rsvector<scalar_type> >
+      M(gmm::mat_nrows(MM), mf_source.nb_basic_dof());
+
+    if (version == 0) mf_source.extend_vector(UU, U);
 
     mti.distribute(extrapolation);
     std::vector<size_type> itab;    
     base_matrix G;
 
-    gmm::clear(M);
-
     /* interpolation */
     dal::bit_vector dof_done; dof_done.add(0, mti.nb_points());
     std::vector<T> val(qdim_s);
@@ -350,7 +392,7 @@ namespace getfem {
     std::vector<size_type> dof_source;
 
     for (dal::bv_visitor cv(mf_source.convex_index()); !cv.finished(); ++cv) {
-      bgeot::pgeometric_trans pgt=msh.trans_of_convex(cv);
+      bgeot::pgeometric_trans pgt = msh.trans_of_convex(cv);
       mti.points_on_convex(cv, itab);
       if (itab.size() == 0) continue;
 
@@ -363,13 +405,14 @@ namespace getfem {
       if (version == 0) {
         coeff.resize(qqdim);
         for (size_type qq=0; qq < qqdim; ++qq) {
-          coeff[qq].resize(mf_source.nb_dof_of_element(cv));
-	  gmm::sub_index SUBI(mf_source.ind_dof_of_element(cv));
+          coeff[qq].resize(mf_source.nb_basic_dof_of_element(cv));
+	  gmm::sub_index SUBI(mf_source.ind_basic_dof_of_element(cv));
           gmm::copy(gmm::sub_vector(U, SUBI), coeff[qq]);
         }
       }
       if (version != 0) {
-	const mesh_fem::ind_dof_ct &idct = mf_source.ind_dof_of_element(cv);
+	const mesh_fem::ind_dof_ct &idct
+	  = mf_source.ind_basic_dof_of_element(cv);
 	dof_source.assign(idct.begin(), idct.end());
       }
       for (size_type i = 0; i < itab.size(); ++i) {
@@ -393,7 +436,7 @@ namespace getfem {
 	    // }
 	  }
 	  else {
-	    base_matrix Mloc(qdim_s, mf_source.nb_dof_of_element(cv));
+	    base_matrix Mloc(qdim_s, mf_source.nb_basic_dof_of_element(cv));
 	    pf_s->interpolation(ctx, Mloc, qdim_s);
 	    for (size_type k=0; k < qdim_s; ++k) {
               for (size_type j=0; j < gmm::mat_ncols(Mloc); ++j)
@@ -408,19 +451,32 @@ namespace getfem {
 	}
       }
     }
-    if (dof_done.card() != 0)
-      GMM_WARNING2("WARNING : in interpolation (different meshes),"
-		   << dof_done.card() << " dof of target mesh_fem have "
-		   << " been missed\nmissing dofs : " << dof_done);
+    if (dof_done.card() != 0) {
+      if (dof_untouched)
+	*dof_untouched = dof_done;
+      else
+	GMM_WARNING2("in interpolation (different meshes),"
+		     << dof_done.card() << " dof of target mesh_fem have "
+		     << " been missed\nmissing dofs : " << dof_done);
+    }
+
+    if (version != 0) {
+      if (mf_source.is_reduced())
+	gmm::mult(M, mf_source.extension_matrix(), MM);
+      else
+	gmm::copy(M, MM);
+    }
+    
   }
 
   template<typename VECTU, typename VECTV>
   void interpolation(const mesh_fem &mf_source, mesh_trans_inv &mti,
-		     const VECTU &U, VECTV &V, bool extrapolation = false) {
+		     const VECTU &U, VECTV &V, int extrapolation = 0,
+		     dal::bit_vector *dof_untouched = 0) {
     base_matrix M;
     GMM_ASSERT1((gmm::vect_size(U) % mf_source.nb_dof()) == 0 &&
-		gmm::vect_size(V)!=0, "Dimensions mismatch");
-    interpolation(mf_source, mti, U, V, M, 0, extrapolation);
+		gmm::vect_size(V)!=0, "Dimension of vector mismatch");
+    interpolation(mf_source, mti, U, V, M, 0, extrapolation, dof_untouched);
   }
 
 
@@ -432,9 +488,16 @@ namespace getfem {
    */
   template<typename VECTU, typename VECTV, typename MAT>
     void interpolation(const mesh_fem &mf_source, const mesh_fem &mf_target,
-		       const VECTU &U, VECTV &V, MAT &M,
-		       int version, bool extrapolation = false,
-		       double EPS = 1E-10) {
+		       const VECTU &U, VECTV &VV, MAT &MM,
+		       int version, int extrapolation,
+		       double EPS) {
+
+    typedef typename gmm::linalg_traits<VECTU>::value_type T;
+    dim_type qqdim = dim_type(gmm::vect_size(U)/mf_source.nb_dof());
+    size_type qqdimt = qqdim * mf_source.get_qdim()/mf_target.get_qdim();
+    std::vector<T> V(mf_target.nb_basic_dof()*qqdimt);
+    gmm::row_matrix<gmm::rsvector<scalar_type> >
+      M(mf_target.nb_basic_dof(), mf_source.nb_dof());
 
     const mesh &msh(mf_source.linked_mesh());
     getfem::mesh_trans_inv mti(msh, EPS);
@@ -450,15 +513,25 @@ namespace getfem {
 		  "Target fem not convenient for interpolation");
     }
     /* initialisation of the mesh_trans_inv */
-    size_type nbpts = mf_target.nb_dof() / qdim_t;
+    size_type nbpts = mf_target.nb_basic_dof() / qdim_t;
     for (size_type i = 0; i < nbpts; ++i)
-      mti.add_point(mf_target.point_of_dof(i * qdim_t));
+      mti.add_point(mf_target.point_of_basic_dof(i * qdim_t));
     interpolation(mf_source, mti, U, V, M, version, extrapolation);
+
+    if (version == 0)
+      mf_target.reduce_vector(V, VV);
+    else {
+      if (mf_target.is_reduced())
+	gmm::mult(mf_target.reduction_matrix(), M, MM); 
+      else
+	gmm::copy(M, MM);
+    }
+
   }
 
   template<typename VECTU, typename VECTV>
   void interpolation(const mesh_fem &mf_source, const mesh_fem &mf_target,
-		     const VECTU &U, VECTV &V, bool extrapolation,
+		     const VECTU &U, VECTV &V, int extrapolation,
 		     double EPS = 1E-10) {
     base_matrix M;
     GMM_ASSERT1((gmm::vect_size(U) % mf_source.nb_dof()) == 0
@@ -473,68 +546,17 @@ namespace getfem {
 
   template<typename MAT>
   void interpolation(const mesh_fem &mf_source, const mesh_fem &mf_target,
-		     MAT &M, bool extrapolation, double EPS = 1E-10) {
+		     MAT &M, int extrapolation, double EPS = 1E-10) {
     GMM_ASSERT1(mf_source.nb_dof() == gmm::mat_ncols(M)
 		&& (gmm::mat_nrows(M) % mf_target.nb_dof()) == 0
 		&& gmm::mat_nrows(M) != 0, "Dimensions mismatch");
     std::vector<scalar_type> U, V;
-    if (&mf_source.linked_mesh() == &mf_target.linked_mesh()) {
+    if (&mf_source.linked_mesh() == &mf_target.linked_mesh())
       interpolation_same_mesh(mf_source, mf_target, U, V, M, 1);
-    }
     else 
       interpolation(mf_source, mf_target, U, V, M, 1, extrapolation, EPS);
   }
 
-  //
-  // Deprecated functions (for version 1.6 -> 1.7)
-  //
-
-  template<typename VECTU, typename VECTV>
-  void interpolation_solution(const mesh_fem &mf_source,
-			      const mesh_fem &mf_target,
-			      const VECTU &U, VECTV &V,
-			      bool extrapolation = false) IS_DEPRECATED;
-
-  template<typename VECTU, typename VECTV>
-  void interpolation_solution(const mesh_fem &mf_source,
-			      const mesh_fem &mf_target,
-			      const VECTU &U, VECTV &V,
-			      bool extrapolation) {
-    base_matrix M;
-    GMM_ASSERT1((gmm::vect_size(U) % mf_source.nb_dof()) == 0
-		&& (gmm::vect_size(V) % mf_target.nb_dof()) == 0
-		&& gmm::vect_size(V) != 0, "Dimensions mismatch");
-    if (&mf_source.linked_mesh() == &mf_target.linked_mesh()) {
-      interpolation_same_mesh(mf_source, mf_target, U, V, M, 0);
-    }
-    else 
-      interpolation(mf_source, mf_target, U, V, M, 0, extrapolation);
-  }
-
-  template<class VECT>
-  void interpolation_solution(const mesh_fem &mf_source,
-			      mesh_trans_inv &mti,
-			      const VECT &U, VECT &V) IS_DEPRECATED;
-
-  template<class VECT>
-  void interpolation_solution(const mesh_fem &mf_source,
-			      mesh_trans_inv &mti,
-			      const VECT &U, VECT &V)
-  { base_matrix M; interpolation_solution(mf_source, mti, U, V, M, 0, false); }
-
-  template<typename VECTU, typename VECTV, typename MAT>
-  void interpolation_solution_same_mesh(const mesh_fem &mf_source,
-					const mesh_fem &mf_target,
-					const VECTU &U, VECTV &V,
-					MAT &M, int version)  IS_DEPRECATED;
-
-  template<typename VECTU, typename VECTV, typename MAT>
-  void interpolation_solution_same_mesh(const mesh_fem &mf_source,
-					const mesh_fem &mf_target,
-					const VECTU &U, VECTV &V,
-					MAT &M, int version)
-  { interpolation_same_mesh(mf_source, mf_target, U, V, M, version); }
-
 }  /* end of namespace getfem.                                             */
 
 
