@@ -17,14 +17,14 @@
 // along  with  this program;  if not, write to the Free Software Foundation,
 // Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// As a special exception, you may use this file as part of a free software
-// library without restriction.  Specifically, if other files instantiate
-// templates or use macros or inline functions from this file, or you compile
-// this file and link it with other files to produce an executable, this
-// file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however
-// invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.
+// As a special exception, you  may use  this file  as it is a part of a free
+// software  library  without  restriction.  Specifically,  if   other  files
+// instantiate  templates  or  use macros or inline functions from this file,
+// or  you compile this  file  and  link  it  with other files  to produce an
+// executable, this file  does  not  by itself cause the resulting executable
+// to be covered  by the GNU Lesser General Public License.  This   exception
+// does not  however  invalidate  any  other  reasons why the executable file
+// might be covered by the GNU Lesser General Public License.
 //
 //===========================================================================
 /**@file getfem_mat_elem_type.h
@@ -67,6 +67,7 @@ namespace getfem {
     mutable std::set<pmat_elem_type> melt_list; /* list of melt that will
 						 * be destroyed by
 						 * ~nonlinear_elem_term */
+    mutable size_type term_num_;
   public :
     virtual const bgeot::multi_index &sizes() const = 0;
     virtual void compute(fem_interpolation_context& /*ctx*/,
@@ -74,6 +75,7 @@ namespace getfem {
     virtual void prepare(fem_interpolation_context& /*ctx*/,
                          size_type /*nl_part*/) {}
     virtual ~nonlinear_elem_term();
+    size_type &term_num(void) const { return term_num_; }
     void register_mat_elem(pmat_elem_type p) /* internal use */
     { melt_list.insert(p); }
   };
