@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //===========================================================================
 //
-// Copyright (C) 2004-2008 Yves Renard
+// Copyright (C) 2004-2010 Yves Renard
 //
 // This file is a part of GETFEM++
 //
@@ -32,7 +32,7 @@
    @file getfem_modeling.h
    @author  Yves Renard <Yves.Renard@insa-lyon.fr>
    @date June 15, 2004.
-   @brief Model Bricks
+   @brief Model Bricks (old system)
    @see model_state
    @see mdbrick_abstract
 */
@@ -110,7 +110,7 @@ namespace getfem {
     T_MATRIX tangent_matrix_;
     C_MATRIX constraints_matrix_;
     VECTOR state_, residual_, constraints_rhs_;
-    long ident_;
+    gmm::uint64_type ident_;
     
     T_MATRIX SM;
     gmm::col_matrix<gmm::rsvector<value_type> > NS; /* constraints nullspace */
@@ -156,8 +156,8 @@ namespace getfem {
     void compute_reduced_system();
     void compute_reduced_residual();
     VECTOR &residual(void) { return residual_; }
-    long ident(void) { return ident_; }
-    void touch(void) { ident_ = context_dependencies::new_ident(); }
+    gmm::uint64_type ident(void) { return ident_; }
+    void touch(void) { ident_ = act_counter(); }
     void adapt_sizes(mdbrick_abstract<model_state> &problem);
     model_state(void) { touch(); }
     model_state(mdbrick_abstract<model_state> &problem)
@@ -1042,7 +1042,7 @@ namespace getfem {
 
     /** Switch between a scalar coefficient, a NxN matrix field (with
 	N = mf_u.linked_mesh().dim()), and a NxNxNxN tensor field. */
-    void set_coeff_dimension(unsigned d) { coeff_.redim(d); }
+    void set_coeff_dimension(unsigned d) { coeff_.redim(d); reshape_coeff(); }
 
     /** Constructor, the default coeff is a scalar equal to one
 	(i.e. it gives the Laplace operator).
@@ -1332,7 +1332,8 @@ namespace getfem {
 	const mesh_fem &mf_u = *(this->mesh_fems[num_fem]);
 	gmm::clear(K);
 	gmm::resize(K, mf_u.nb_dof(), mf_u.nb_dof());
-	asm_qu_term(K, *(this->mesh_ims[0]), mf_u, Q().mf(), Q().get(), boundary);
+	asm_qu_term(K, *(this->mesh_ims[0]), mf_u, Q().mf(), Q().get(),
+		    mf_u.linked_mesh().get_mpi_sub_region(boundary));
 	K_uptodate = true;
 	this->parameters_set_uptodate();
       }
@@ -1578,6 +1579,11 @@ namespace getfem {
 
   public :
 
+    // For Tomas Ligursky !!
+    const VECTOR &get_CRHS(void) const { return CRHS; }
+    size_type first_ind(void)
+    { return this->first_index()+sub_problem.nb_dof(); }
+
     /* provide access to the value of the multipliers only for multiplier
      * option (could be extended ?).
      */
@@ -1809,7 +1815,12 @@ namespace getfem {
  	mfdata_set = true;
       }
       size_type nd = mf_u().nb_dof();
-      dal::bit_vector dof_on_bound = mf_mult->dof_on_set(boundary);
+      dal::bit_vector dof_on_bound;
+      if (mf_mult->is_reduced())
+	dof_on_bound.add(0, mf_mult->nb_dof());
+      else
+	dof_on_bound = mf_mult->basic_dof_on_region(boundary);
+
       size_type nb_const = dof_on_bound.card();
       std::vector<size_type> ind_ct;
       for (dal::bv_visitor i(dof_on_bound); !i.finished(); ++i)
@@ -1925,7 +1936,11 @@ namespace getfem {
  	mfdata_set = true;
       }
       size_type nd = mf_u().nb_dof();
-      dal::bit_vector dof_on_bound = mf_mult.dof_on_set(boundary);
+      dal::bit_vector dof_on_bound;
+      if (mf_mult.is_reduced())
+	dof_on_bound.add(0, nd);
+      else
+	dof_on_bound = mf_mult.basic_dof_on_region(boundary);
       size_type nb_const = dof_on_bound.card();
       std::vector<size_type> ind_ct;
       for (dal::bv_visitor i(dof_on_bound); !i.finished(); ++i)
@@ -1966,7 +1981,7 @@ namespace getfem {
 
     /** Switch between a scalar coefficient, a N vector field or a NxN
 	matrix field. */
-    void set_coeff_dimension(unsigned d) { R_.redim(d); }
+    void set_coeff_dimension(unsigned d) { R_.redim(d); reshape_coeff(); }
 
     /** Constructor which does not define the rhs (i.e. which sets an
      *	homogeneous Dirichlet condition)
@@ -2066,7 +2081,8 @@ namespace getfem {
 	R tol=gmm::mat_maxnorm(M)*gmm::default_tol(value_type())*R(100);
 	gmm::clean(M, tol);
 	std::vector<size_type> ind_ct;
-	dal::bit_vector nn = mf_u().dof_on_set(boundary);
+	GMM_ASSERT1(!mf_u().is_reduced(), "to be adapted");
+	dal::bit_vector nn = mf_u().basic_dof_on_region(boundary);
 	// The following filter is not sufficient for an arbitrary matrix field
 	// H for the multipliers version. To be ameliorated.
 	
@@ -2211,6 +2227,7 @@ namespace getfem {
     size_type num_fem;
     value_type Mcoef, Kcoef;
     std::set<size_type> boundary_sup;
+    bool only_vert_sup; // Uggly
     bool M_uptodate;
 
     virtual void proper_update(void) {
@@ -2225,13 +2242,15 @@ namespace getfem {
 			    RHO_.get());
 
       if (!(boundary_sup.empty())) {
+	GMM_ASSERT1(!mf_u->is_reduced(), "To be adapted");
+
 	gmm::unsorted_sub_index SUBS;
 	std::vector<size_type> ind;
 	dal::bit_vector ind_set;
 	
 	for (std::set<size_type>::const_iterator it = boundary_sup.begin();
 	     it != boundary_sup.end(); ++it) {
-	  ind_set = ind_set | mf_u->dof_on_set(*it);
+	  ind_set = ind_set | mf_u->basic_dof_on_region(*it);
 	}
 
 	VECTOR V(mf_u->nb_dof()), MV(mf_u->nb_dof()); 
@@ -2268,6 +2287,7 @@ namespace getfem {
 
     /* valid only for lagrange FEMs */
     void redistribute_mass(const dal::bit_vector &redistributed_dof) {
+      GMM_ASSERT1(!mf_u->is_reduced(), "To be adapted");
       size_type N = mf_u->linked_mesh().dim();
       size_type Qdim = mf_u->get_qdim();
       size_type nn = mf_u->nb_dof() / Qdim;
@@ -2279,7 +2299,7 @@ namespace getfem {
 
       /* get the non-zero coef as a vector */
       size_type nz = gmm::nnz(M0);
-      gmm::array1D_reference<value_type*> vM0(M0.pr, nz);
+      gmm::array1D_reference<value_type*> vM0(&M0.pr[0], nz);
 
       //dal::bit_vector removed; removed.sup(0, nz);
       size_type nbmult = 1+N+N*(N+1)/2;
@@ -2291,8 +2311,8 @@ namespace getfem {
       for (size_type j=0, ii=0; j < gmm::mat_ncols(M0); ++j) {
 	for (size_type ir=0; ir < M0.jc[j+1] - M0.jc[j]; ++ir, ++ii) {
 	  size_type i=M0.ir[ii];
-	  const base_node Pi = mf_u->point_of_dof(i * Qdim);
-	  const base_node Pj = mf_u->point_of_dof(j * Qdim);
+	  const base_node Pi = mf_u->point_of_basic_dof(i * Qdim);
+	  const base_node Pj = mf_u->point_of_basic_dof(j * Qdim);
 	  C(0, ii) = 1; // X'MX 
 	  d[0] += vM0[ii];
 	
@@ -2330,7 +2350,7 @@ namespace getfem {
       gmm::mult(gmm::transposed(C), gmm::scaled(L, -1), vM0); gmm::add(F, vM0);
 
 
-      /* force the symmetricity */
+      /* enforce the symmetry */
       gmm::copy(vM0, F);
       for (size_type j=0, ii=0; j < gmm::mat_ncols(M0); ++j) {
 	for (size_type ir=0; ir < M0.jc[j+1] - M0.jc[j]; ++ir, ++ii) {
@@ -2346,10 +2366,12 @@ namespace getfem {
 
       /* write back the mass matrix */
 
-      for (unsigned k=0; k < Qdim; ++k) {
-	gmm::sub_slice IND(k, nn, Qdim);
-	gmm::copy(M0, gmm::sub_matrix(M_, IND, IND));
-      }
+      for (unsigned k=0; k < Qdim; ++k)
+	if (!only_vert_sup || k == Qdim-1) {
+	  cout << "write back for " << k << endl;
+	  gmm::sub_slice IND(k, nn, Qdim);
+	  gmm::copy(M0, gmm::sub_matrix(M_, IND, IND));
+	}
 
       /* some sanity checks */
       for (unsigned i=0; i < gmm::mat_nrows(M0); ++i) {
@@ -2363,8 +2385,8 @@ namespace getfem {
 	for (size_type ir=0; ir < M0.jc[j+1] - M0.jc[j]; ++ir, ++ii) {
 	  size_type i=M0.ir[ii];
 	  if (i > j) continue; // deal with upper triangle only
-	  const base_node Pi = mf_u->point_of_dof(i * Qdim);
-	  const base_node Pj = mf_u->point_of_dof(j * Qdim);
+	  const base_node Pi = mf_u->point_of_basic_dof(i * Qdim);
+	  const base_node Pj = mf_u->point_of_basic_dof(j * Qdim);
 	  e[0] += vM0[ii];
 	
 	  for (unsigned k=0; k < N; ++k) {
@@ -2383,6 +2405,9 @@ namespace getfem {
 
   public :
 
+    mdbrick_parameter<VECTOR> &rho() { return RHO_; M_uptodate = true; }
+    const mdbrick_parameter<VECTOR> &rho() const { return RHO_; }
+
     virtual void do_compute_tangent_matrix(MODEL_STATE &MS, size_type i0,
 					   size_type) {
       gmm::sub_interval
@@ -2425,7 +2450,8 @@ namespace getfem {
       this->force_update();
     }
 
-    void no_mass_on_boundary(size_type b) {
+    void no_mass_on_boundary(size_type b, bool only_vert_sup_ = false) {
+      only_vert_sup = only_vert_sup_;
       if (boundary_sup.find(b) == boundary_sup.end()) {
 	boundary_sup.insert(b);
 	this->force_update();
