@@ -35,7 +35,6 @@
 
 namespace getfem {
 
-  typedef dal::naming_system<virtual_fem>::param_list fem_param_list;
 
   const base_matrix& fem_interpolation_context::M() const {
     if (gmm::mat_nrows(M_) == 0) {
@@ -62,7 +61,7 @@ namespace getfem {
       pf()->real_base_value(*this, t);
     else {
       base_tensor u;
-      if (have_pfp()) {
+      if (have_pfp() && ii() != size_type(-1)) {
 	switch(pf()->vectorial_type()) {
 	case virtual_fem::VECTORIAL_PRIMAL_TYPE:
 	  t.mat_transp_reduction(pfp_->val(ii()), K(), 1); break;
@@ -91,7 +90,7 @@ namespace getfem {
       pf()->real_grad_base_value(*this, t);
     else {
       base_tensor u;
-      if (have_pfp()) {
+      if (have_pfp() && ii() != size_type(-1)) {
 	switch(pf()->vectorial_type()) {
 	case virtual_fem::VECTORIAL_PRIMAL_TYPE:
 	  u.mat_transp_reduction(pfp_->grad(ii()), B(), 2);
@@ -126,7 +125,8 @@ namespace getfem {
       pf()->real_hess_base_value(*this, t);
     else {
       base_tensor tt;
-      if (have_pfp()) tt = pfp()->hess(ii()); else pf()->hess_base_value(xref(), tt);
+      if (have_pfp() && ii() != size_type(-1))
+	tt = pfp()->hess(ii()); else pf()->hess_base_value(xref(), tt);
 
       switch(pf()->vectorial_type()) {
       case virtual_fem::VECTORIAL_PRIMAL_TYPE:
@@ -916,9 +916,6 @@ namespace getfem {
     return fem_descriptor(name.str());
   }
 
-  static void read_poly(bgeot::base_poly &p, int d, const char *s) 
-  { p = bgeot::read_base_poly(short_type(d), s); }
-
   /* ******************************************************************** */
   /*	P1 NON CONFORMING (dim 2)                                         */
   /* ******************************************************************** */
@@ -1452,7 +1449,7 @@ namespace getfem {
     if (!(pgt->is_linear())) gmm::mult(G, pgp->grad(3), K);
     if (N == 1) M(3, 3) = K(0,0);
     else M(3, 3) = gmm::mat_euclidean_norm(K)
-      * gmm::sgn(gmm::vect_sp(gmm::mat_row(K, 0), r));
+      * gmm::sgn(gmm::vect_sp(gmm::mat_col(K, 0), r));
   }
 
   // Hermite element on the segment. when the real element lies in
@@ -1471,13 +1468,13 @@ namespace getfem {
     pt[0] = 0.0; add_node(lagrange_dof(1), pt);
     read_poly(base_[0], 1, "(1 - x)^2*(2*x + 1)");
 
-    pt[0] = 0.0; add_node(derivative_dof(1, 0), pt, dal::bit_vector());
+    pt[0] = 0.0; add_node(derivative_dof(1, 0), pt);
     read_poly(base_[1], 1, "x*(x - 1)*(x - 1)");
 
     pt[0] = 1.0; add_node(lagrange_dof(1), pt);
     read_poly(base_[2], 1, "x*x*(3  - 2*x)");
 
-    pt[0] = 1.0; add_node(derivative_dof(1, 0), pt, dal::bit_vector());
+    pt[0] = 1.0; add_node(derivative_dof(1, 0), pt);
     read_poly(base_[3], 1, "x*x*(x - 1)");
   }
 
@@ -2151,6 +2148,12 @@ namespace getfem {
       shorter_name_of_method(p);
   }
 
+  // allows the add of a fem.
+  void add_fem_name(std::string name,
+		    dal::naming_system<virtual_fem>::pfunction f) {
+    dal::singleton<fem_naming_system>::instance().add_suffix(name, f);
+  }
+
   /* ******************************************************************** */
   /*    Aliases functions                                                 */
   /* ******************************************************************** */
