@@ -123,7 +123,7 @@ namespace getfem {
   void fem_level_set::real_base_value(const fem_interpolation_context &c,
 				      base_tensor &t, bool) const {
     bgeot::multi_index mi(2);
-    mi[1] = target_dim(); mi[0] = nb_base(0);
+    mi[1] = target_dim(); mi[0] = short_type(nb_base(0));
     t.adjust_sizes(mi);
     base_tensor::iterator it = t.begin();
     fem_interpolation_context c0 = c;
@@ -150,7 +150,8 @@ namespace getfem {
   void fem_level_set::real_grad_base_value(const fem_interpolation_context &c,
 					   base_tensor &t, bool) const {
     bgeot::multi_index mi(3);
-    mi[2] = c.N(); mi[1] = target_dim(); mi[0] = nb_base(0);
+    mi[2] = short_type(c.N()); mi[1] = target_dim();
+    mi[0] = short_type(nb_base(0));
     t.adjust_sizes(mi);
     fem_interpolation_context c0 = c;
     if (c0.have_pfp())
@@ -181,7 +182,8 @@ namespace getfem {
   void fem_level_set::real_hess_base_value(const fem_interpolation_context &c,
 				  base_tensor &t, bool) const {
     bgeot::multi_index mi(4);
-    mi[3] = mi[2] = c.N(); mi[1] = target_dim(); mi[0] = nb_base(0);
+    mi[3] = mi[2] = short_type(c.N()); mi[1] = target_dim();
+    mi[0] = short_type(nb_base(0));
     t.adjust_sizes(mi);
     fem_interpolation_context c0 = c;
     if (c0.have_pfp())
