@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //===========================================================================
 //
-// Copyright (C) 1999-2008 Yves Renard
+// Copyright (C) 1999-2009 Yves Renard
 //
 // This file is a part of GETFEM++
 //
@@ -25,9 +25,55 @@
 #include "getfem/getfem_mesh_fem.h"
 
 namespace getfem {
+
+  void mesh_fem::update_from_context(void) const {
+    for (dal::bv_visitor i(fe_convex); !i.finished(); ++i) {
+      if (linked_mesh_->convex_index().is_in(i)) {
+	if (v_num_update < linked_mesh_->convex_version_number(i)) {
+	  if (auto_add_elt_pf != 0)
+	    const_cast<mesh_fem *>(this)
+	      ->set_finite_element(i, auto_add_elt_pf);
+	  else if (auto_add_elt_K != dim_type(-1)) {
+	    if (auto_add_elt_disc)
+	      const_cast<mesh_fem *>(this)
+		->set_classical_discontinuous_finite_element(i,auto_add_elt_K,
+							 auto_add_elt_alpha);
+	    else
+	      const_cast<mesh_fem *>(this)
+		->set_classical_finite_element(i, auto_add_elt_K);
+	  }
+	  else
+	    const_cast<mesh_fem *>(this)
+	      ->set_finite_element(i, 0);
+	}
+      }
+      else const_cast<mesh_fem *>(this)->set_finite_element(i, 0);
+    }
+    for (dal::bv_visitor i(linked_mesh_->convex_index());
+	 !i.finished(); ++i) {
+      if (!fe_convex.is_in(i)
+	  && v_num_update < linked_mesh_->convex_version_number(i)) {
+	if (auto_add_elt_pf != 0)
+	  const_cast<mesh_fem *>(this)
+	    ->set_finite_element(i, auto_add_elt_pf);
+	else if (auto_add_elt_K != dim_type(-1)) {
+	  if (auto_add_elt_disc)
+	    const_cast<mesh_fem *>(this)
+	      ->set_classical_discontinuous_finite_element(i, auto_add_elt_K,
+							   auto_add_elt_alpha);
+	  else
+	    const_cast<mesh_fem *>(this)
+	      ->set_classical_finite_element(i, auto_add_elt_K);
+	}
+      }
+    }
+    if (!dof_enumeration_made) enumerate_dof();
+    v_num = v_num_update = act_counter();
+  }
   
-  dal::bit_vector mesh_fem::dof_on_region(const mesh_region &b) const {
-    if (!dof_enumeration_made) this->enumerate_dof();
+  dal::bit_vector mesh_fem::basic_dof_on_region(const mesh_region &b) const {
+    // cout << "dof on region for " << b << endl;
+    context_check(); if (!dof_enumeration_made) this->enumerate_dof();
     dal::bit_vector res;
     for (getfem::mr_visitor v(b,linked_mesh()); !v.finished(); ++v) {
       size_type cv = v.cv();
@@ -37,7 +83,7 @@ namespace getfem {
 	  size_type nbb =
 	    dof_structure.structure_of_convex(cv)->nb_points_of_face(f);
 	  for (size_type i = 0; i < nbb; ++i) {
-	    size_type n = Qdim / fem_of_element(cv)->target_dim();
+	    size_type n = Qdim/fem_of_element(cv)->target_dim();
 	    for (size_type ll = 0; ll < n; ++ll)
 	      res.add(dof_structure.ind_points_of_face_of_convex(cv,f)[i]+ll);
 	  }
@@ -45,7 +91,7 @@ namespace getfem {
 	  size_type nbb =
 	    dof_structure.structure_of_convex(cv)->nb_points();
 	  for (size_type i = 0; i < nbb; ++i) {
-	    size_type n = Qdim / fem_of_element(cv)->target_dim();
+	    size_type n = Qdim/fem_of_element(cv)->target_dim();
 	    for (size_type ll = 0; ll < n; ++ll)
 	      res.add(dof_structure.ind_points_of_convex(cv)[i]+ll);
 	  }
@@ -54,50 +100,35 @@ namespace getfem {
     }
     return res;
   }
-  
-  void mesh_fem::receipt(const MESH_CLEAR &) { clear(); }
-  void mesh_fem::receipt(const MESH_DELETE &) { clear(); }
-  void mesh_fem::receipt(const MESH_SUP_CONVEX &m) { 
-    if (fe_convex[m.icv])
-      { fe_convex[m.icv] = false; dof_enumeration_made = false; }
-  }
-  void mesh_fem::receipt(const MESH_ADD_CONVEX &m) {
-    if (auto_add_elt_K != size_type(-1)) {
-      pfem pf = getfem::classical_fem(linked_mesh().trans_of_convex(m.icv), 
-				      short_type(auto_add_elt_K));
-      set_finite_element(m.icv, pf);
-    }
-  }
-  void mesh_fem::receipt(const MESH_SWAP_CONVEX &m) { 
-    fe_convex.swap(m.icv1, m.icv2);
-    f_elems.swap(m.icv1, m.icv2);
+
+  template <typename V>
+  static void add_e_line__(const V &v, dal::bit_vector &r) {
+    typedef typename gmm::linalg_traits<V>::value_type T;
+    typename gmm::linalg_traits<V>::const_iterator it = gmm::vect_begin(v);
+    typename gmm::linalg_traits<V>::const_iterator ite = gmm::vect_end(v);
+    for (; it != ite; ++it) if (*it != T(0)) r.add(it.index());
   }
 
-  /* keep the FEM and dof_partition when the mesh is refined */
-  void mesh_fem::receipt(const MESH_REFINE_CONVEX &m) { 
-    if (m.is_refine) {
-      if (fe_convex[m.icv])
-	for (size_type i = 0; i < m.sub_cv_list.size(); ++i) {
-	  f_elems[m.sub_cv_list[i]] = f_elems[m.icv];
-	  fe_convex.add(m.sub_cv_list[i]);
-	}
-      for (size_type i = 0; i < m.sub_cv_list.size(); ++i) {
-	set_dof_partition(m.sub_cv_list[i], get_dof_partition(m.icv));
-      }
-    }
-    else if (fe_convex[m.sub_cv_list[0]]) {
-      f_elems[m.icv] = f_elems[m.sub_cv_list[0]];
-      fe_convex.add(m.icv);
-      set_dof_partition(m.icv, get_dof_partition(m.sub_cv_list[0]));
+  dal::bit_vector mesh_fem::dof_on_region(const mesh_region &b) const {
+    dal::bit_vector res = basic_dof_on_region(b);
+    if (is_reduced()) {
+      dal::bit_vector basic = res;
+      res.clear();
+      for (dal::bv_visitor i(basic); !i.finished(); ++i)
+	add_e_line__(gmm::mat_row(E_, i), res);
     }
+    return res;
   }
+
    
   void mesh_fem::set_finite_element(size_type cv, pfem pf) {
+    GMM_ASSERT1(linked_mesh_ != 0, "Uninitialized mesh_fem");
+    context_check();
     if (pf == 0) {
       if (fe_convex.is_in(cv)) {
 	fe_convex.sup(cv);
 	dof_enumeration_made = false;
-	touch();
+	touch(); v_num = act_counter();
       }
     }
     else {
@@ -114,7 +145,7 @@ namespace getfem {
 	fe_convex.add(cv);
 	f_elems[cv] = pf;
 	dof_enumeration_made = false;  
-	touch();
+	touch(); v_num = act_counter();
       }
     }
   }
@@ -125,8 +156,15 @@ namespace getfem {
   }
 
   void mesh_fem::set_finite_element(pfem ppf)
-  { set_finite_element(linked_mesh().convex_index(), ppf); }
+  { set_finite_element(linked_mesh().convex_index(), ppf); set_auto_add(ppf); }
   
+  void mesh_fem::set_classical_finite_element(size_type cv,
+					      dim_type fem_degree) {
+    pfem pf = getfem::classical_fem(linked_mesh().trans_of_convex(cv), 
+				    fem_degree);
+    set_finite_element(cv, pf);
+  }
+
   void mesh_fem::set_classical_finite_element(const dal::bit_vector &cvs, 
 					      dim_type fem_degree) {
     for (dal::bv_visitor cv(cvs); !cv.finished(); ++cv) {
@@ -136,8 +174,17 @@ namespace getfem {
     }
   }
 
-  void mesh_fem::set_classical_finite_element(dim_type fem_degree)
-  { set_classical_finite_element(linked_mesh().convex_index(), fem_degree); }
+  void mesh_fem::set_classical_finite_element(dim_type fem_degree) {
+    set_classical_finite_element(linked_mesh().convex_index(), fem_degree);
+    set_auto_add(fem_degree, false);
+  }
+
+  void mesh_fem::set_classical_discontinuous_finite_element
+  (size_type cv, dim_type fem_degree, scalar_type alpha) {
+    pfem pf = getfem::classical_discontinuous_fem
+      (linked_mesh().trans_of_convex(cv), fem_degree, alpha);
+    set_finite_element(cv, pf);
+  }
 
   void mesh_fem::set_classical_discontinuous_finite_element
   (const dal::bit_vector &cvs, dim_type fem_degree, scalar_type alpha) {
@@ -152,17 +199,19 @@ namespace getfem {
   (dim_type fem_degree, scalar_type alpha) { 
     set_classical_discontinuous_finite_element(linked_mesh().convex_index(),
 					       fem_degree,alpha);
+    set_auto_add(fem_degree, true, alpha);
   }
 
-  base_node mesh_fem::point_of_dof(size_type cv, size_type i) const {
+  base_node mesh_fem::point_of_basic_dof(size_type cv, size_type i) const {
+    context_check(); if (!dof_enumeration_made) enumerate_dof();
     pfem pf = f_elems[cv];
     return linked_mesh().trans_of_convex(cv)->transform
       (pf->node_of_dof(cv, i * pf->target_dim() / Qdim),
        linked_mesh().points_of_convex(cv));
   }
 
-  base_node mesh_fem::point_of_dof(size_type d) const {
-    if (!dof_enumeration_made) enumerate_dof();
+  base_node mesh_fem::point_of_basic_dof(size_type d) const {
+    context_check(); if (!dof_enumeration_made) enumerate_dof();
     for (size_type i = d; i != d - Qdim && i != size_type(-1); --i) {
       size_type cv = dof_structure.first_convex_of_point(i);
       if (cv != size_type(-1)) {
@@ -175,15 +224,21 @@ namespace getfem {
     GMM_ASSERT1(false, "Inexistent dof");
   }
 
-  dim_type mesh_fem::dof_qdim(size_type d) const {
-    if (!dof_enumeration_made) enumerate_dof();
-    size_type cv = first_convex_of_dof(d);
-    GMM_ASSERT1(cv != size_type(-1), "Inexistent dof");
-    size_type tdim = f_elems[cv]->target_dim();
-    return dim_type(dof_structure.ind_in_convex_of_point(cv, d) % (Qdim/tdim));
+  dim_type mesh_fem::basic_dof_qdim(size_type d) const {
+    context_check(); if (!dof_enumeration_made) enumerate_dof();
+    for (size_type i = d; i != d - Qdim && i != size_type(-1); --i) {
+      size_type cv = dof_structure.first_convex_of_point(i);   
+      if (cv != size_type(-1)) { 
+	size_type tdim = f_elems[cv]->target_dim();
+	return dim_type((d-i) / tdim);
+      }
+    }
+    GMM_ASSERT1(false, "Inexistent dof");
+    return 0;
   }
 
-  size_type mesh_fem::first_convex_of_dof(size_type d) const {
+  size_type mesh_fem::first_convex_of_basic_dof(size_type d) const {
+    context_check();
     for (size_type i = d; i != d - Qdim && i != size_type(-1); --i) {
       size_type j = dof_structure.first_convex_of_point(i);
       if (j != size_type(-1)) return j;
@@ -191,7 +246,7 @@ namespace getfem {
     return size_type(-1);
   }
 
-  const mesh::ind_cv_ct &mesh_fem::convex_to_dof(size_type d) const {
+  const mesh::ind_cv_ct &mesh_fem::convex_to_basic_dof(size_type d) const {
     for (size_type i = d; i != d - Qdim && i != size_type(-1); --i) {
       size_type j = dof_structure.first_convex_of_point(i);
       if (j != size_type(-1)) return dof_structure.convex_to_point(i);
@@ -220,6 +275,8 @@ namespace getfem {
 
   /// Enumeration of dofs
   void mesh_fem::enumerate_dof(void) const {
+    GMM_ASSERT1(linked_mesh_ != 0, "Uninitialized mesh_fem");
+    context_check();
     if (fe_convex.card() == 0) {
       dof_enumeration_made = true;
       nb_total_dof = 0;
@@ -294,32 +351,65 @@ namespace getfem {
     nb_total_dof = nbdof;
   }
 
+  void mesh_fem::reduce_to_basic_dof(const dal::bit_vector &kept_dof) {
+    gmm::row_matrix<gmm::rsvector<scalar_type> >
+      RR(kept_dof.card(), nb_basic_dof());
+    size_type j = 0;
+    for (dal::bv_visitor i(kept_dof); !i.finished(); ++i, ++j)
+      RR(j, i) = scalar_type(1);
+    set_reduction_matrices(RR, gmm::transposed(RR));
+  }
+
+  void mesh_fem::reduce_to_basic_dof(const std::set<size_type> &kept_dof) {
+    gmm::row_matrix<gmm::rsvector<scalar_type> >
+      RR(kept_dof.size(), nb_basic_dof());
+    size_type j = 0;
+    for (std::set<size_type>::iterator it = kept_dof.begin();
+	 it != kept_dof.end(); ++it, ++j)
+      RR(j, *it) = scalar_type(1);
+    set_reduction_matrices(RR, gmm::transposed(RR));
+  }
+
   void mesh_fem::clear(void) {
     fe_convex.clear();
     dof_enumeration_made = false;
-    touch();
+    touch(); v_num = act_counter();
     dof_structure.clear();
+    use_reduction = false;
+    R_ = REDUCTION_MATRIX();
+    E_ = EXTENSION_MATRIX();
+  }
+
+  void mesh_fem::init_with_mesh(const mesh &me, dim_type Q) {
+    GMM_ASSERT1(linked_mesh_ == 0, "Mesh level set already initialized");
+    dof_enumeration_made = false;
+    auto_add_elt_pf = 0;
+    auto_add_elt_K = dim_type(-1);
+    Qdim = Q;
+    QdimM = 1;
+    QdimN = 1;
+    linked_mesh_ = &me;
+    use_reduction = false;
+    this->add_dependency(me);
+    v_num = v_num_update = act_counter();
   }
 
   mesh_fem::mesh_fem(const mesh &me, dim_type Q)
-    : dof_enumeration_made(false), auto_add_elt_K(size_type(-1)), 
-      Qdim(Q), QdimM(1), QdimN(1) {
-    linked_mesh_ = &me;
-    this->add_dependency(me);
-    add_sender(me.lmsg_sender(), *this,
-	       mask(MESH_CLEAR::ID)   | mask(MESH_SUP_CONVEX::ID) |
-	       mask(MESH_SWAP_CONVEX::ID) | mask(MESH_DELETE::ID) |
-	       mask(MESH_ADD_CONVEX::ID)|
-	       mask(MESH_REFINE_CONVEX::ID));
-  }
+  { linked_mesh_ = 0; init_with_mesh(me, Q); }
 
-  mesh_fem::~mesh_fem() {}
+  mesh_fem::mesh_fem(void) 
+  { linked_mesh_ = 0; dof_enumeration_made = false; }
+
+  mesh_fem::~mesh_fem() { }
 
   void mesh_fem::read_from_file(std::istream &ist) {
+    GMM_ASSERT1(linked_mesh_ != 0, "Uninitialized mesh_fem");
+    gmm::stream_standard_locale sl(ist);
     dal::bit_vector npt;
     dal::dynamic_array<double> tmpv;
     std::string tmp, tmp2;
     bool dof_read = false;
+    gmm::col_matrix< gmm::wsvector<scalar_type> > RR, EE;
     ist.precision(16);
     clear();
     ist.seekg(0);ist.clear();
@@ -353,7 +443,8 @@ namespace getfem {
 	  ist >> bgeot::skip("END DOF_PARTITION");
 	} else if (bgeot::casecmp(tmp, "DOF_ENUMERATION") == 0) {
 	  dal::bit_vector doflst;
-	  dof_structure.clear(); dof_enumeration_made = false; touch();
+	  dof_structure.clear(); dof_enumeration_made = false;
+	  touch(); v_num = act_counter();
 	  while (true) {
 	    bgeot::get_token(ist, tmp);
 	    if (bgeot::casecmp(tmp, "END")==0) { 
@@ -364,9 +455,10 @@ namespace getfem {
 	    size_type ic = atoi(tmp.c_str());
 	    std::vector<size_type> tab;
 	    if (convex_index().is_in(ic) && tmp.size() &&
-		isdigit(tmp[0]) && tmp2 == ":") { // && tmp[tmp.size()-1] == ':') {
-	      tab.resize(nb_dof_of_element(ic));
-	      for (size_type i=0; i < fem_of_element(ic)->nb_dof(ic); i++) {
+		isdigit(tmp[0]) && tmp2 == ":") {
+	      tab.resize(nb_basic_dof_of_element(ic));
+	      for (size_type i=0; i < fem_of_element(ic)->nb_dof(ic);
+		   i++) {
 		ist >> tab[i];
 		for (size_type q=0; q < size_type(get_qdim())
 		       / fem_of_element(ic)->target_dim(); ++q)
@@ -383,10 +475,71 @@ namespace getfem {
 	  } 
 	  dof_read = true;
 	  this->dof_enumeration_made = true;
-	  touch();
+	  touch(); v_num = act_counter();
 	  this->nb_total_dof = doflst.card();
 	  ist >> bgeot::skip("DOF_ENUMERATION");
-	} else if (tmp.size())
+	} else if  (bgeot::casecmp(tmp, "REDUCTION_MATRIX")==0) {
+	  bgeot::get_token(ist, tmp);
+	  GMM_ASSERT1(bgeot::casecmp(tmp, "NROWS")==0,
+		      "Missing number of rows");
+	  size_type nrows; ist >> nrows;
+	  bgeot::get_token(ist, tmp);
+	  GMM_ASSERT1(bgeot::casecmp(tmp, "NCOLS")==0,
+		      "Missing number of columns");
+	  size_type ncols; ist >> ncols;
+	  bgeot::get_token(ist, tmp);
+	  GMM_ASSERT1(bgeot::casecmp(tmp, "NNZ")==0,
+		      "Missing number of nonzero elements");
+	  size_type nnz; ist >> nnz;
+	  gmm::clear(RR); gmm::resize(RR, nrows, ncols);
+	  for (size_type i = 0; i < ncols; ++i) {
+	    bgeot::get_token(ist, tmp);
+	    GMM_ASSERT1(bgeot::casecmp(tmp, "COL")==0,
+			"Missing some columns");
+	    size_type nnz_col; ist >> nnz_col;
+	    for (size_type j=0; j<nnz_col; ++j) {
+	      size_type ind; ist >> ind;
+	      scalar_type val; ist >> val;
+	      RR(ind, i) = val; // can be optimized using a csc matrix
+	    }
+	  }
+	  R_ = REDUCTION_MATRIX(nrows, ncols);
+	  gmm::copy(RR, R_);
+	  use_reduction = true;
+	  ist >> bgeot::skip("END");
+	  ist >> bgeot::skip("REDUCTION_MATRIX");
+	} else if  (bgeot::casecmp(tmp, "EXTENSION_MATRIX")==0) {
+	  bgeot::get_token(ist, tmp);
+	  GMM_ASSERT1(bgeot::casecmp(tmp, "NROWS")==0,
+		      "Missing number of rows");
+	  size_type nrows; ist >> nrows;
+	  bgeot::get_token(ist, tmp);
+	  GMM_ASSERT1(bgeot::casecmp(tmp, "NCOLS")==0,
+		      "Missing number of columns");
+	  size_type ncols; ist >> ncols;
+	  bgeot::get_token(ist, tmp);
+	  GMM_ASSERT1(bgeot::casecmp(tmp, "NNZ")==0,
+		      "Missing number of nonzero elements");
+	  size_type nnz; ist >> nnz;
+	  gmm::clear(EE); gmm::resize(EE, nrows, ncols);
+	  for (size_type i = 0; i < nrows; ++i) {
+	    bgeot::get_token(ist, tmp);
+	    GMM_ASSERT1(bgeot::casecmp(tmp, "ROW")==0,
+			"Missing some rows");
+	    size_type nnz_row; ist >> nnz_row;
+	    for (size_type j=0; j < nnz_row; ++j) {
+	      size_type ind; ist >> ind;
+	      scalar_type val; ist >> val;
+	      EE(i, ind) = val; // can be optimized using a csc matrix
+	    }
+	  }
+	  E_ = EXTENSION_MATRIX(nrows, ncols);
+	  gmm::copy(EE, E_);
+	  use_reduction = true;
+	  ist >> bgeot::skip("END");
+	  ist >> bgeot::skip("EXTENSION_MATRIX");
+	}
+	else if (tmp.size())
 	  GMM_ASSERT1(false, "Syntax error in file at token '"
 		      << tmp << "' [pos=" << std::streamoff(ist.tellg())
 		      << "]");
@@ -412,8 +565,40 @@ namespace getfem {
     read_from_file(o);
   }
 
-  void mesh_fem::write_to_file(std::ostream &ost) const {
-    ost << '\n' << "BEGIN MESH_FEM" << '\n' << '\n';
+  template<typename VECT> static void
+  write_col(std::ostream &ost, const VECT &v) {
+    typename gmm::linalg_traits<VECT>::const_iterator it = v.begin();
+    ost << gmm::nnz(v);
+    for (; it != v.end(); ++it)
+      ost << " " << it.index() << " " << *it;
+    ost << "\n";
+  }
+
+  void mesh_fem::write_reduction_matrices_to_file(std::ostream &ost) const {
+    if (use_reduction) {
+      ost.precision(16);
+      ost << " BEGIN REDUCTION_MATRIX " << '\n';
+      ost << "  NROWS " <<  gmm::mat_nrows(R_) << '\n'; 
+      ost << "  NCOLS " <<  gmm::mat_ncols(R_) << '\n';
+      ost << "  NNZ " << gmm::nnz(R_) << '\n';
+      for (size_type i = 0; i < gmm::mat_ncols(R_); ++i) {
+	ost << "  COL ";
+	write_col(ost, gmm::mat_col(R_, i));
+      }
+      ost << " END REDUCTION_MATRIX " << '\n';
+      ost << " BEGIN EXTENSION_MATRIX " << '\n';
+      ost << "  NROWS " <<  gmm::mat_nrows(E_) << '\n'; 
+      ost << "  NCOLS " <<  gmm::mat_ncols(E_) << '\n';
+      ost << "  NNZ " << gmm::nnz(E_) << '\n';
+      for (size_type i = 0; i < gmm::mat_nrows(E_); ++i) {
+	ost << "  ROW ";
+	write_col(ost, gmm::mat_row(E_, i));
+      }
+      ost << " END EXTENSION_MATRIX " << '\n';
+    }
+  }
+
+  void mesh_fem::write_basic_to_file(std::ostream &ost) const {
     ost << "QDIM " << size_type(get_qdim()) << '\n';
     for (dal::bv_visitor cv(convex_index()); !cv.finished(); ++cv) {
       ost << " CONVEX " << cv;
@@ -433,17 +618,25 @@ namespace getfem {
     ost << " BEGIN DOF_ENUMERATION " << '\n';
     for (dal::bv_visitor cv(convex_index()); !cv.finished(); ++cv) {
       ost << "  " << cv << ": ";
-      ind_dof_ct::const_iterator it = ind_dof_of_element(cv).begin();
-      while (it != ind_dof_of_element(cv).end()) {
+      ind_dof_ct::const_iterator it = ind_basic_dof_of_element(cv).begin();
+      while (it != ind_basic_dof_of_element(cv).end()) {
 	ost << " " << *it;
 	// skip repeated dofs for "pseudo" vector elements
 	for (size_type i=0;
-	     i < size_type(get_qdim())/fem_of_element(cv)->target_dim(); ++i)
-	  ++it;
+	     i < size_type(get_qdim())/fem_of_element(cv)->target_dim();
+	     ++i) ++it;
       }
       ost << '\n';
     }
     ost << " END DOF_ENUMERATION " << '\n';
+  }
+
+  void mesh_fem::write_to_file(std::ostream &ost) const {
+    context_check();
+    gmm::stream_standard_locale sl(ost);
+    ost << '\n' << "BEGIN MESH_FEM" << '\n' << '\n';
+    write_basic_to_file(ost);
+    write_reduction_matrices_to_file(ost);
     ost << "END MESH_FEM" << '\n';
   }
 
@@ -456,56 +649,57 @@ namespace getfem {
     write_to_file(o);
   }
 
-  struct mf__key_ {
+  struct mf__key_ : public context_dependencies {
     const mesh *pmesh;
-    dim_type order;
-    mf__key_(const mesh &msh, dim_type o) : pmesh(&msh),order(o) {}
+    dim_type order, qdim;
+    mf__key_(const mesh &msh, dim_type o, dim_type q)
+      : pmesh(&msh), order(o), qdim(q)  { add_dependency(msh); }
     bool operator <(const mf__key_ &a) const {
-    if (pmesh < a.pmesh) return true; else
-      if (a.pmesh < pmesh) return false; else
-	if (order < a.order) return true; else return false;
+      if (pmesh < a.pmesh) return true;
+      else if (pmesh > a.pmesh) return false;
+      else if (order < a.order) return true; 
+      else if (order > a.order) return false;
+      else if (qdim < a.qdim) return true;
+      return false;
     }
+    void update_from_context(void) const {}
+    mf__key_(const mf__key_ &mfk) {
+      pmesh = mfk.pmesh; order = mfk.order; qdim = mfk.qdim;
+      add_dependency(*pmesh);
+    }
+  private :
+    mf__key_& operator=(const mf__key_ &mfk);
   };
 
 
-  class classical_mesh_fem_pool : public mesh_receiver {
+  class classical_mesh_fem_pool {
 
     typedef const mesh_fem * pmesh_fem;
     typedef std::map<mf__key_, pmesh_fem> mesh_fem_tab;
 
     mesh_fem_tab mfs;
 
-    // void receipt(const MESH_CLEAR &) {}
-    void receipt(const MESH_DELETE &M) {
-      for (mesh_fem_tab::iterator it = mfs.begin(); it != mfs.end(); ) {
-	mesh_fem_tab::iterator it2 = it; it2 ++;
-	if (it->first.pmesh == M.msh) mfs.erase(it);
-	it = it2;
-      }
-    }
-//     void receipt(const MESH_ADD_CONVEX &m) {}
-//     void receipt(const MESH_SUP_CONVEX &m) {}
-//     void receipt(const MESH_SWAP_CONVEX &m) {}
-
   public :
 
-    const mesh_fem &operator()(const mesh &msh, dim_type o) {
+    const mesh_fem &operator()(const mesh &msh, dim_type o, dim_type qdim) {
+      mesh_fem_tab::iterator itt = mfs.begin(), itn = mfs.begin();
+      if (itn != mfs.end()) itn++;
+      while (itt != mfs.end()) {
+	if (!(itt->first.is_context_valid()))
+	  { delete itt->second; mfs.erase(itt); }
+	itt=itn;
+	if (itn != mfs.end()) itn++;
+      }
 
-      mf__key_ key(msh, o);
+      mf__key_ key(msh, o, qdim);
       mesh_fem_tab::iterator it = mfs.find(key);
       assert(it == mfs.end() || it->second->is_context_valid());
       
       if (it == mfs.end()) {
-	// the list of mesh pointers should be sorted ...
-	for (mesh_fem_tab::iterator itt = mfs.begin(); itt != mfs.end(); ++itt)
-	  if (itt->first.pmesh == &msh) goto nothing_to_do;
-	add_sender(msh.lmsg_sender(), *this, mask(MESH_DELETE::ID));
-	
-      nothing_to_do :
-	
 	mesh_fem *pmf = new mesh_fem(msh);
-	pmf->set_auto_add(o);
 	pmf->set_classical_finite_element(o);
+	pmf->set_auto_add(o, false);
+	pmf->set_qdim(qdim);
 	return *(mfs[key] = pmf);
       }
       else return *(it->second);
@@ -514,10 +708,10 @@ namespace getfem {
   };
 
   const mesh_fem &classical_mesh_fem(const mesh &msh,
-				     dim_type order) {
+				     dim_type order, dim_type qdim) {
     classical_mesh_fem_pool &pool
       = dal::singleton<classical_mesh_fem_pool>::instance();
-    return pool(msh, order);
+    return pool(msh, order, qdim);
   }
 
   struct dummy_mesh_fem_ {
@@ -525,7 +719,9 @@ namespace getfem {
     mesh_fem mf;
     dummy_mesh_fem_() : mf(m) {}
   };
-  const mesh_fem &dummy_mesh_fem(void) { return dal::singleton<dummy_mesh_fem_>::instance().mf; }
+
+  const mesh_fem &dummy_mesh_fem(void)
+  { return dal::singleton<dummy_mesh_fem_>::instance().mf; }
 
 
 
