@@ -1,24 +1,32 @@
 // -*- c++ -*- (enables emacs c++ mode)
-//========================================================================
+//===========================================================================
 //
-// Copyright (C) 2003-2007 Yves Renard
+// Copyright (C) 2003-2008 Yves Renard
 //
 // This file is a part of GETFEM++
 //
-// Getfem++ is free software; you can redistribute it and/or modify
-// it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; version 2.1 of the License.
+// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+// under  the  terms  of the  GNU  Lesser General Public License as published
+// by  the  Free Software Foundation;  either version 2.1 of the License,  or
+// (at your option) any later version.
+// This program  is  distributed  in  the  hope  that it will be useful,  but
+// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+// License for more details.
+// You  should  have received a copy of the GNU Lesser General Public License
+// along  with  this program;  if not, write to the Free Software Foundation,
+// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU Lesser General Public License for more details.
-// You should have received a copy of the GNU Lesser General Public
-// License along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
-// USA.
+// As a special exception, you  may use  this file  as it is a part of a free
+// software  library  without  restriction.  Specifically,  if   other  files
+// instantiate  templates  or  use macros or inline functions from this file,
+// or  you compile this  file  and  link  it  with other files  to produce an
+// executable, this file  does  not  by itself cause the resulting executable
+// to be covered  by the GNU Lesser General Public License.  This   exception
+// does not  however  invalidate  any  other  reasons why the executable file
+// might be covered by the GNU Lesser General Public License.
 //
-//========================================================================
+//===========================================================================
 
 /**@file gmm_inoutput.h
    @author Yves Renard <Yves.Renard@insa-lyon.fr>
@@ -150,9 +158,10 @@ namespace gmm {
       memset(Title, 0, sizeof Title); 
     }
     char *getline(char *buf) { 
-      fgets(buf, BUFSIZ, f); ++lcount;
+      char *p = fgets(buf, BUFSIZ, f); ++lcount;
       int s = SECURE_NONCHAR_SSCANF(buf,"%*s");
-      GMM_ASSERT1(s >= 0, "blank line in HB file at line " << lcount);
+      GMM_ASSERT1(s >= 0 && p != 0,
+		  "blank line in HB file at line " << lcount);
       return buf;
     }
 
@@ -165,11 +174,11 @@ namespace gmm {
       SECURE_STRNCPY(s, 100, p, len); s[len] = 0;
       if ( Valflag != 'F' && !strchr(s,'E')) {
 	/* insert a char prefix for exp */
-	int last = strlen(s);
+	int last = int(strlen(s));
 	for (int j=last+1;j>=0;j--) {
 	  s[j] = s[j-1];
 	  if ( s[j] == '+' || s[j] == '-' ) {
-	    s[j-1] = Valflag;                    
+	    s[j-1] = char(Valflag);                    
 	    break;
 	  }
 	}
@@ -202,6 +211,8 @@ namespace gmm {
       int Valperline, Valwidth, Valprec, Nentries;
       int Valflag;           /* Indicates 'E','D', or 'F' float format */
       char line[BUFSIZ];
+      gmm::standard_locale sl;
+
 
       /*  Parse the array input formats from Line 3 of HB file  */
       ParseIfmt(Ptrfmt,&Ptrperline,&Ptrwidth);
@@ -209,8 +220,6 @@ namespace gmm {
       if ( Type[0] != 'P' ) {          /* Skip if pattern only  */
 	ParseRfmt(Valfmt,&Valperline,&Valwidth,&Valprec,&Valflag);
       }
-
-      cout << "Valwidth = " << Valwidth << endl; getchar();
     
       /*  Read column pointer array:   */
       offset = 0;         /* if base 0 storage is declared (via macro def),  */
@@ -287,7 +296,7 @@ namespace gmm {
 	       &Neltvl) < 1)
 #endif
       IOHBTerminate("Invalid Type info, line 3 of Harwell-Boeing file.\n");
-    for (size_type i = 0; i < 3; ++i) Type[i] = toupper(Type[i]);
+    for (size_type i = 0; i < 3; ++i) Type[i] = char(toupper(Type[i]));
     
       /*  Fourth line:  */
 #ifdef GMM_SECURE_CRT
@@ -326,13 +335,11 @@ namespace gmm {
 		"Bad HB matrix format (file contains a REAL matrix)");
     GMM_ASSERT1(is_complex_double__(T()) || Type[0] != 'C',
 		"Bad HB matrix format (file contains a COMPLEX matrix)");
-    if (A.pr) { delete[] A.pr; delete[] A.ir; delete[] A.jc; }
     A.nc = ncols(); A.nr = nrows();
-    A.pr = 0;
-    A.jc = new IND_TYPE[ncols()+1];
-    A.ir = new IND_TYPE[nnz()];
-    A.pr = new T[nnz()];
-    readHB_data(A.jc, A.ir, (double*)A.pr);
+    A.jc.resize(ncols()+1);
+    A.ir.resize(nnz());
+    A.pr.resize(nnz());
+    readHB_data(&A.jc[0], &A.ir[0], (double*)&A.pr[0]);
     for (int i = 0; i <= ncols(); ++i) { A.jc[i] += shift; A.jc[i] -= 1; }
     for (int i = 0; i < nnz(); ++i)    { A.ir[i] += shift; A.ir[i] -= 1; }
   }
@@ -374,6 +381,8 @@ namespace gmm {
     int Valperline, Valwidth, Valprec;
     int Valflag;           /* Indicates 'E','D', or 'F' float format */
     char pformat[16],iformat[16],vformat[19],rformat[19];
+    //    char *pValflag, *pRhsflag;
+    gmm::standard_locale sl;
     
     if ( Type[0] == 'C' )
       { nvalentries = 2*nz; nrhsentries = 2*M; }
@@ -400,7 +409,10 @@ namespace gmm {
     if ( Type[0] != 'P' ) {          /* Skip if pattern only  */
       if ( Valfmt == NULL ) Valfmt = "(4E21.13)";
       ParseRfmt(Valfmt, &Valperline, &Valwidth, &Valprec, &Valflag);
-      if (Valflag == 'D') *strchr(Valfmt,'D') = 'E';
+//       if (Valflag == 'D') {
+//         pValflag = (char *) strchr(Valfmt,'D');
+//         *pValflag = 'E';
+//       }
       if (Valflag == 'F')
 	SECURE_SPRINTF2(vformat, sizeof(vformat), "%% %d.%df", Valwidth,
 			Valprec);
@@ -418,7 +430,10 @@ namespace gmm {
 	SECURE_SPRINTF2(rformat,sizeof(rformat), "%% %d.%df",Rhswidth,Rhsprec);
       else
 	SECURE_SPRINTF2(rformat,sizeof(rformat), "%% %d.%dE",Rhswidth,Rhsprec);
-      if (Rhsflag == 'D') *strchr(Rhsfmt,'D') = 'E';
+//       if (Valflag == 'D') {
+//         pRhsflag = (char *) strchr(Rhsfmt,'D');
+//         *pRhsflag = 'E';
+//       }
       rhscrd = nrhsentries/Rhsperline; 
       if ( nrhsentries%Rhsperline != 0) rhscrd++;
       if ( Rhstype[1] == 'G' ) rhscrd+=rhscrd;
@@ -518,16 +533,18 @@ namespace gmm {
   template <typename T, int shift> void
   HarwellBoeing_IO::write(const char *filename,
 			  const csc_matrix<T, shift>& A) {
-    write(filename, csc_matrix_ref<T*, unsigned*, unsigned *, shift>
-	  (A.pr, A.ir, A.jc, A.nr, A.nc));
+    write(filename, csc_matrix_ref<const T*, const unsigned*,
+	  const unsigned *, shift>
+	  (&A.pr[0], &A.ir[0], &A.jc[0], A.nr, A.nc));
   }
 
   template <typename T, int shift> void
   HarwellBoeing_IO::write(const char *filename,
 			  const csc_matrix<T, shift>& A,
 			  const std::vector<T> &rhs) {
-    write(filename, csc_matrix_ref<T*, unsigned*, unsigned *, shift>
-	  (A.pr, A.ir, A.jc, A.nr, A.nc), rhs);
+    write(filename, csc_matrix_ref<const T*, const unsigned*,
+	  const unsigned *, shift>
+	  (&A.pr[0], &A.ir[0], &A.jc[0], A.nr, A.nc), rhs);
   }
 
   template <typename T, typename INDI, typename INDJ, int shift> void
@@ -538,7 +555,7 @@ namespace gmm {
       if (mat_nrows(A) == mat_ncols(A)) t = "CUA"; else t = "CRA";
     else
       if (mat_nrows(A) == mat_ncols(A)) t = "RUA"; else t = "RRA";
-    writeHB_mat_double(filename, mat_nrows(A), mat_ncols(A),
+    writeHB_mat_double(filename, int(mat_nrows(A)), int(mat_ncols(A)),
 		       A.jc[mat_ncols(A)], A.jc, A.ir,
 		       (const double *)A.pr,
 		       0, 0, 0, 0, "GETFEM++ CSC MATRIX", "CSCMAT",
@@ -555,7 +572,7 @@ namespace gmm {
     else
       if (mat_nrows(A) == mat_ncols(A)) t = "RUA"; else t = "RRA";
     int Nrhs = gmm::vect_size(rhs) / mat_nrows(A);
-    writeHB_mat_double(filename, mat_nrows(A), mat_ncols(A),
+    writeHB_mat_double(filename, int(mat_nrows(A)), int(mat_ncols(A)),
 		       A.jc[mat_ncols(A)], A.jc, A.ir,
 		       (const double *)A.pr,
 		       Nrhs, (const double *)(&rhs[0]), 0, 0,
@@ -799,6 +816,7 @@ namespace gmm {
     char data_type[MM_MAX_TOKEN_LENGTH];
     char storage_scheme[MM_MAX_TOKEN_LENGTH];
     char *p;
+    gmm::standard_locale sl;
     /*    int ret_code; */
     
     mm_clear_typecode(matcode);  
@@ -817,10 +835,10 @@ namespace gmm {
 #endif
       return MM_PREMATURE_EOF;
 
-    for (p=mtx; *p!='\0'; *p=tolower(*p),p++);  /* convert to lower case */
-    for (p=crd; *p!='\0'; *p=tolower(*p),p++);  
-    for (p=data_type; *p!='\0'; *p=tolower(*p),p++);
-    for (p=storage_scheme; *p!='\0'; *p=tolower(*p),p++);
+    for (p=mtx; *p!='\0'; *p=char(tolower(*p)),p++) {};  /* convert to lower case */
+    for (p=crd; *p!='\0'; *p=char(tolower(*p)),p++) {};  
+    for (p=data_type; *p!='\0'; *p=char(tolower(*p)),p++) {};
+    for (p=storage_scheme; *p!='\0'; *p=char(tolower(*p)),p++) {};
 
     /* check for banner */
     if (strncmp(banner, MatrixMarketBanner, strlen(MatrixMarketBanner)) != 0)
@@ -1032,6 +1050,7 @@ namespace gmm {
 
 
   inline void MatrixMarket_IO::open(const char *filename) {
+    gmm::standard_locale sl;
     if (f) { fclose(f); }
     SECURE_FOPEN(&f, filename, "r");
     GMM_ASSERT1(f, "Sorry, cannot open file " << filename);
@@ -1053,6 +1072,7 @@ namespace gmm {
   }
 
   template <typename Matrix> void MatrixMarket_IO::read(Matrix &A) {
+    gmm::standard_locale sl;
     typedef typename linalg_traits<Matrix>::value_type T;
     GMM_ASSERT1(f, "no file opened!");
     GMM_ASSERT1(!is_complex_double__(T()) || isComplex,
@@ -1067,18 +1087,35 @@ namespace gmm {
     mm_read_mtx_crd_data(f, row, col, nz, &I[0], &J[0],
 			 (double*)&PR[0], matcode);
     
-    for (size_type i = 0; i < size_type(nz); ++i) A(I[i]-1, J[i]-1) = PR[i];
+    for (size_type i = 0; i < size_type(nz); ++i) {
+        A(I[i]-1, J[i]-1) = PR[i];
+
+        // FIXED MM Format
+        if (mm_is_hermitian(matcode) && (I[i] != J[i]) ) {
+            A(J[i]-1, I[i]-1) = gmm::conj(PR[i]);
+        }
+
+        if (mm_is_symmetric(matcode) && (I[i] != J[i]) ) {
+            A(J[i]-1, I[i]-1) = PR[i];
+        }
+
+        if (mm_is_skew(matcode) && (I[i] != J[i]) ) {
+            A(J[i]-1, I[i]-1) = -PR[i];
+        }
+    }
   }
 
   template <typename T, int shift> void 
   MatrixMarket_IO::write(const char *filename, const csc_matrix<T, shift>& A) {
-    write(filename, csc_matrix_ref<T*,unsigned*,unsigned*,shift>
-	  (A.pr, A.ir, A.jc, A.nr, A.nc));
+    write(filename, csc_matrix_ref<const T*, const unsigned*,
+	  const unsigned*,shift>
+	  (&A.pr[0], &A.ir[0], &A.jc[0], A.nr, A.nc));
   }
 
   template <typename T, typename INDI, typename INDJ, int shift> void 
   MatrixMarket_IO::write(const char *filename, 
 			 const csc_matrix_ref<T*, INDI*, INDJ*, shift>& A) {
+    gmm::standard_locale sl;
     static MM_typecode t1 = {'M', 'C', 'R', 'G'};
     static MM_typecode t2 = {'M', 'C', 'C', 'G'};
     MM_typecode t;
@@ -1090,11 +1127,11 @@ namespace gmm {
     for (size_type j=0; j < mat_ncols(A); ++j) {      
       for (size_type i = A.jc[j]; i < A.jc[j+1]; ++i) {
 	I[i] = A.ir[i] + 1 - shift;
-	J[i] = j + 1;
+	J[i] = int(j + 1);
       }
     }
-    mm_write_mtx_crd(filename, mat_nrows(A), mat_ncols(A),
-		     nz, &I[0], &J[0], (const double *)A.pr, t);
+    mm_write_mtx_crd(filename, int(mat_nrows(A)), int(mat_ncols(A)),
+		     int(nz), &I[0], &J[0], (const double *)A.pr, t);
   }
 
 
@@ -1107,14 +1144,14 @@ namespace gmm {
   }
 
   template<typename VEC> static void vecsave(std::string fname, const VEC& V) {
-    std::ofstream f(fname.c_str()); f.precision(16);
+    std::ofstream f(fname.c_str()); f.precision(16); f.imbue(std::locale("C"));
     for (size_type i=0; i < gmm::vect_size(V); ++i) f << V[i] << "\n"; 
   } 
 
   template<typename VEC> static void vecload(std::string fname,
 					     const VEC& V_) {
     VEC &V(const_cast<VEC&>(V_));
-    std::ifstream f(fname.c_str());
+    std::ifstream f(fname.c_str()); f.imbue(std::locale("C"));
     for (size_type i=0; i < gmm::vect_size(V); ++i) f >> V[i]; 
   }
 }
