@@ -212,7 +212,8 @@ struct elastostatic_problem {
 
   std::string datafilename;
   bgeot::md_param PARAM;
-
+  
+  void select_boundaries(void);
   bool solve(plain_vector &U);
   void init(void);
   void compute_error(plain_vector &U);
@@ -220,6 +221,24 @@ struct elastostatic_problem {
 			       mf_rhs(mesh),mf_p(mesh) {}
 };
 
+/* Selects the boundaries */
+
+void elastostatic_problem::select_boundaries(void) {
+  size_type N = mesh.dim();
+  getfem::mesh_region border_faces;
+  getfem::outer_faces_of_mesh(mesh, border_faces);
+  for (getfem::mr_visitor i(border_faces); !i.finished(); ++i) {
+    base_node un = mesh.normal_of_face_of_convex(i.cv(), i.f());
+    un /= gmm::vect_norm2(un);
+    if (gmm::abs(un[N-1] - 1.0) < 0.5) { // new Neumann face
+      mesh.region(NEUMANN_BOUNDARY_NUM).add(i.cv(), i.f());
+    } else {
+      mesh.region(DIRICHLET_BOUNDARY_NUM).add(i.cv(), i.f());
+    }
+  }
+}
+
+
 /* Read parameters from the .param file, build the mesh, set finite element
  * and integration methods and selects the boundaries.
  */
@@ -279,24 +298,22 @@ void elastostatic_problem::init(void) {
   getfem::pintegration_method ppi = 
     getfem::int_method_descriptor(INTEGRATION);
 
-  mim.set_integration_method(mesh.convex_index(), ppi);
-  mf_u.set_finite_element(mesh.convex_index(), pf_u);
+  mim.set_integration_method(ppi);
+  mf_u.set_finite_element(pf_u);
 
   std::string dirichlet_fem_name = PARAM.string_value("DIRICHLET_FEM_TYPE");
   if (dirichlet_fem_name.size() == 0)
-    mf_mult.set_finite_element(mesh.convex_index(), pf_u);
+    mf_mult.set_finite_element(pf_u);
   else {
     cout << "DIRICHLET_FEM_TYPE="  << dirichlet_fem_name << "\n";
-    mf_mult.set_finite_element(mesh.convex_index(), 
-			       getfem::fem_descriptor(dirichlet_fem_name));
+    mf_mult.set_finite_element(getfem::fem_descriptor(dirichlet_fem_name));
   }
 
   mixed_pressure =
     (PARAM.int_value("MIXED_PRESSURE","Mixed version or not.") != 0);
   if (mixed_pressure) {
     std::string FEM_TYPE_P  = PARAM.string_value("FEM_TYPE_P","FEM name P");
-    mf_p.set_finite_element(mesh.convex_index(),
-			    getfem::fem_descriptor(FEM_TYPE_P));
+    mf_p.set_finite_element(getfem::fem_descriptor(FEM_TYPE_P));
   }
 
   /* set the finite element on mf_rhs (same as mf_u is DATA_FEM_TYPE is
@@ -306,26 +323,15 @@ void elastostatic_problem::init(void) {
     GMM_ASSERT1(pf_u->is_lagrange(), "You are using a non-lagrange FEM. "
 		<< "In that case you need to set "
 		<< "DATA_FEM_TYPE in the .param file");
-    mf_rhs.set_finite_element(mesh.convex_index(), pf_u);
+    mf_rhs.set_finite_element(pf_u);
   } else {
-    mf_rhs.set_finite_element(mesh.convex_index(), 
-			      getfem::fem_descriptor(data_fem_name));
+    mf_rhs.set_finite_element(getfem::fem_descriptor(data_fem_name));
   }
   
   /* set boundary conditions
    * (Neuman on the upper face, Dirichlet elsewhere) */
   cout << "Selecting Neumann and Dirichlet boundaries\n";
-  getfem::mesh_region border_faces;
-  getfem::outer_faces_of_mesh(mesh, border_faces);
-  for (getfem::mr_visitor i(border_faces); !i.finished(); ++i) {
-    base_node un = mesh.normal_of_face_of_convex(i.cv(), i.f());
-    un /= gmm::vect_norm2(un);
-    if (gmm::abs(un[N-1] - 1.0) < 0.5) { // new Neumann face
-      mesh.region(NEUMANN_BOUNDARY_NUM).add(i.cv(), i.f());
-    } else {
-      mesh.region(DIRICHLET_BOUNDARY_NUM).add(i.cv(), i.f());
-    }
-  }
+  select_boundaries();
 
 #if GETFEM_PARA_LEVEL > 1
   
@@ -345,10 +351,10 @@ void elastostatic_problem::init(void) {
 /* compute the error with respect to the exact solution */
 void elastostatic_problem::compute_error(plain_vector &U) {
   size_type N = mesh.dim();
-  std::vector<scalar_type> V(mf_rhs.nb_dof()*N);
+  std::vector<scalar_type> V(mf_rhs.nb_basic_dof()*N);
   getfem::interpolation(mf_u, mf_rhs, U, V);
-  for (size_type i = 0; i < mf_rhs.nb_dof(); ++i) {
-    gmm::add(gmm::scaled(sol_u(mf_rhs.point_of_dof(i)), -1.0),
+  for (size_type i = 0; i < mf_rhs.nb_basic_dof(); ++i) {
+    gmm::add(gmm::scaled(sol_u(mf_rhs.point_of_basic_dof(i)), -1.0),
 	     gmm::sub_vector(V, gmm::sub_interval(i*N, N)));
   }
 
@@ -384,63 +390,75 @@ bool elastostatic_problem::solve(plain_vector &U) {
   if (mixed_pressure) cout << "Number of dof for P: " << mf_p.nb_dof() << endl;
   cout << "Number of dof for u: " << mf_u.nb_dof() << endl;
 
+  getfem::model model;
+
+  // Main unknown of the problem.
+  model.add_fem_variable("u", mf_u);
+
   // Linearized elasticity brick.
-  getfem::mdbrick_isotropic_linearized_elasticity<>
-    ELAS(mim, mf_u, mixed_pressure ? 0.0 : lambda, mu);
+  model.add_initialized_scalar_data("lambda", mixed_pressure ? 0.0 : lambda);
+  model.add_initialized_scalar_data("mu", mu);
+  getfem::add_isotropic_linearized_elasticity_brick
+    (model, mim, "u", "lambda", "mu");
 
-  std::auto_ptr<getfem::mdbrick_abstract<> > INCOMP;  
+  // Linearized incompressibility condition brick.
   if (mixed_pressure) {
-    getfem::mdbrick_linear_incomp<> *incomp =
-      new getfem::mdbrick_linear_incomp<>(ELAS, mf_p);
-    incomp->penalization_coeff().set(1.0/lambda);
-    INCOMP.reset(incomp);
+    model.add_initialized_scalar_data("incomp_coeff", 1.0/lambda);
+    model.add_fem_variable("p", mf_p); // Adding the pressure as a variable
+    add_linear_incompressibility
+      (model, mim, "u", "p", size_type(-1), "incomp_coeff");
   }
-  getfem::mdbrick_abstract<> *pINCOMP;
-  if (mixed_pressure) pINCOMP = INCOMP.get(); else pINCOMP = &ELAS;
-  
-  // Volumic source term brick.
-  getfem::mdbrick_source_term<> VOL_F(*pINCOMP);
-  
-  // Neumann condition brick.
-  getfem::mdbrick_normal_source_term<>
-    NEUMANN(VOL_F, NEUMANN_BOUNDARY_NUM);
 
-  // Dirichlet condition brick.
-  getfem::mdbrick_Dirichlet<> final_model(NEUMANN, DIRICHLET_BOUNDARY_NUM,
-					  mf_mult);
-  final_model.set_constraints_type(dirichlet_version);
- 
+  // Volumic source term.
+  std::vector<scalar_type> F(mf_rhs.nb_dof()*N);
+  getfem::interpolation_function(mf_rhs, F, sol_f);
+  model.add_initialized_fem_data("VolumicData", mf_rhs, F);
+  getfem::add_source_term_brick(model, mim, "u", "VolumicData");
+    
+  // Neumann condition.
+  gmm::resize(F, mf_rhs.nb_dof()*N*N);
+  getfem::interpolation_function(mf_rhs, F, sol_sigma, NEUMANN_BOUNDARY_NUM);
+  model.add_initialized_fem_data("NeumannData", mf_rhs, F);
+  getfem::add_normal_source_term_brick
+    (model, mim, "u", "NeumannData", NEUMANN_BOUNDARY_NUM);
+
+  // Dirichlet condition.
+  gmm::resize(F, mf_rhs.nb_dof()*N);
+  getfem::interpolation_function(mf_rhs, F, sol_u);
+  model.add_initialized_fem_data("DirichletData", mf_rhs, F);
+  getfem::add_Dirichlet_condition_with_multipliers
+    (model, mim, "u", mf_u, DIRICHLET_BOUNDARY_NUM, "DirichletData");
 
-  cout << "Total number of variables : " << final_model.nb_dof() << endl;
-  getfem::standard_model_state MS(final_model);
   gmm::iteration iter(residual, 1, 40000);
 #if GETFEM_PARA_LEVEL > 1
   double t_init=MPI_Wtime();
 #endif
   dal::bit_vector cvref;
-
+  
   do { // solve with optional refinement
+    
+    cout << "Total number of variables : " << model.nb_dof() << endl;
 
     // Defining the volumic source term.
     size_type nb_dof_rhs = mf_rhs.nb_dof();
-    plain_vector F(nb_dof_rhs * N);
+    gmm::resize(F, nb_dof_rhs * N);
     getfem::interpolation_function(mf_rhs, F, sol_f);
-    VOL_F.source_term().set(mf_rhs, F);
+    gmm::copy(F, model.set_real_variable("VolumicData"));
 
     // Defining the Neumann source term.
     gmm::resize(F, nb_dof_rhs * N * N);
     getfem::interpolation_function(mf_rhs, F, sol_sigma, NEUMANN_BOUNDARY_NUM);
-    NEUMANN.normal_source_term().set(mf_rhs, F);
+    gmm::copy(F, model.set_real_variable("NeumannData"));
 
     // Defining the Dirichlet condition value.
     gmm::resize(F, nb_dof_rhs * N);
     getfem::interpolation_function(mf_rhs, F, sol_u, DIRICHLET_BOUNDARY_NUM);
-    final_model.rhs().set(mf_rhs, F);
-
+    gmm::copy(F, model.set_real_variable("DirichletData"));
+    
     iter.init();
-    getfem::standard_solve(MS, final_model, iter);
+    getfem::standard_solve(model, iter);
     gmm::resize(U, mf_u.nb_dof());
-    gmm::copy(ELAS.get_solution(MS), U);
+    gmm::copy(model.real_variable("u"), U);
 
     if (refine) {
       plain_vector ERR(mesh.convex_index().last_true()+1);
@@ -481,7 +499,7 @@ int main(int argc, char *argv[]) {
   GMM_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
   FE_ENABLE_EXCEPT;        // Enable floating point exception for Nan.
 
-  try {
+  //  try {
 
 
 
@@ -524,12 +542,11 @@ int main(int argc, char *argv[]) {
       exp.exporting(p.mf_u); 
       exp.write_point_data(p.mf_u, U, "elastostatic_displacement");
       cout << "export done, you can view the data file with (for example)\n"
-	"mayavi -d " << p.datafilename << ".vtk -f ExtractVectorNorm -f "
-	"WarpVector -m BandedSurfaceMap -m Outline\n";
+	"mayavi2 -d " << p.datafilename << ".vtk -f ExtractVectorNorm -f "
+	"WarpVector -m Surface -m Outline\n";
     }
 
-  }
-  GMM_STANDARD_CATCH_ERROR;
+    //  } GMM_STANDARD_CATCH_ERROR;
 
   GETFEM_MPI_FINALIZE;
 
