@@ -31,8 +31,8 @@
    a pde directly with the assembly procedures.
 */
 
-#include "getfem/getfem_assembling.h" /* assembly methods (and comp. of norms) */
-#include "getfem/getfem_export.h"   /* export functions (save solutions in a file) */
+#include "getfem/getfem_assembling.h"
+#include "getfem/getfem_export.h"
 #include "getfem/getfem_regular_meshes.h"
 #include "getfem/getfem_derivatives.h"
 #include "getfem/getfem_superlu.h"
@@ -50,6 +50,7 @@ using bgeot::size_type;   /* = unsigned long */
 typedef gmm::rsvector<scalar_type> sparse_vector_type;
 typedef gmm::row_matrix<sparse_vector_type> sparse_matrix_type;
 typedef gmm::col_matrix<sparse_vector_type> col_sparse_matrix_type;
+typedef std::vector<scalar_type> plain_vector;
 
 /* Definitions for the exact solution of the Laplacian problem,
  *  i.e. Delta(sol_u) + sol_f = 0
@@ -78,7 +79,7 @@ struct laplacian_problem {
   getfem::mesh_fem mf_coef; /* the mesh_fem to represent pde coefficients    */
 
   scalar_type residual;        /* max residual for the iterative solvers */
-  size_type est_degree, N;
+  size_type N;
   bool gen_dirichlet;
 
   sparse_matrix_type SM;     /* stiffness matrix.                           */
@@ -134,14 +135,14 @@ void laplacian_problem::init(void) {
 
   datafilename = PARAM.string_value("ROOTFILENAME","Base name of data files.");
   scalar_type FT = PARAM.real_value("FT", "parameter for exact solution");
-  residual = PARAM.real_value("RESIDUAL"); if (residual == 0.) residual = 1e-10;
+  residual = PARAM.real_value("RESIDUAL");
+  if (residual == 0.) residual = 1e-10;
   sol_K.resize(N);
   for (size_type j = 0; j < N; j++)
     sol_K[j] = ((j & 1) == 0) ? FT : -FT;
 
   /* set the finite element on the mf_u */
   getfem::pfem pf_u = getfem::fem_descriptor(FEM_TYPE);
-  est_degree = pf_u->estimated_degree();
   getfem::pintegration_method ppi = getfem::int_method_descriptor(INTEGRATION);
 
   mim.set_integration_method(mesh.convex_index(), ppi);
@@ -253,7 +254,11 @@ void laplacian_problem::assembly(void) {
   }
 }
 
+
 bool laplacian_problem::solve(void) {
+
+  // see_schmidt(SM, U, B);
+
   cout << "Compute preconditionner\n";
   gmm::iteration iter(residual, 1, 40000);
   double time = gmm::uclock_sec();
@@ -295,10 +300,10 @@ bool laplacian_problem::solve(void) {
 
 /* compute the error with respect to the exact solution */
 void laplacian_problem::compute_error() {
-  std::vector<scalar_type> V(mf_rhs.nb_dof());
+  std::vector<scalar_type> V(mf_rhs.nb_basic_dof());
   getfem::interpolation(mf_u, mf_rhs, U, V);
-  for (size_type i = 0; i < mf_rhs.nb_dof(); ++i)
-    V[i] -= sol_u(mf_rhs.point_of_dof(i));
+  for (size_type i = 0; i < mf_rhs.nb_basic_dof(); ++i)
+    V[i] -= sol_u(mf_rhs.point_of_basic_dof(i));
   cout.precision(16);
   cout << "L2 error = " << getfem::asm_L2_norm(mim, mf_rhs, V) << endl
        << "H1 error = " << getfem::asm_H1_norm(mim, mf_rhs, V) << endl
