@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //===========================================================================
 //
-// Copyright (C) 2007-2008 Yves Renard, Julien Pommier.
+// Copyright (C) 2007-2010 Yves Renard, Julien Pommier.
 //
 // This file is a part of GETFEM++
 //
@@ -128,7 +128,7 @@ struct g_params {
 g_params param;
 
 void g_params::init(int argc, char *argv[]) {
-  PARAM.add_int_param("NX", 50);
+  PARAM.add_int_param("NX", 25);
   PARAM.add_int_param("NDIM", 2);
   PARAM.add_int_param("MESH_TYPE", 0);
   PARAM.add_int_param("K", 3);
@@ -185,10 +185,10 @@ namespace getfem {
 	  base_tensor::iterator p = t.begin();
 	  for (size_type i = 0; i < nbd2; i++)
 	    {
-	      size_type dof2 = mfdata.ind_dof_of_element(cv)[i];
+	      size_type dof2 = mfdata.ind_basic_dof_of_element(cv)[i];
 	      for (size_type j = 0; j < nbd1; j++, ++p)
 		{
-		  size_type dof1 = mf.ind_dof_of_element(cv)[j];
+		  size_type dof1 = mf.ind_basic_dof_of_element(cv)[j];
 		  for (size_type k = 0; k < N; k++) {
 		    B[dof1*N + k] += F[dof2*N+k]*(*p);
 		  }
@@ -232,10 +232,10 @@ namespace getfem {
 	base_tensor::iterator p = t.begin();
 	for (size_type i = 0; i < nbd2; i++)
 	  {
-	    size_type dof2 = mfdata.ind_dof_of_element(cv)[i];
+	    size_type dof2 = mfdata.ind_basic_dof_of_element(cv)[i];
 	    for (size_type j = 0; j < nbd1; j++, ++p)
 	      {
-		size_type dof1 = mf.ind_dof_of_element(cv)[j];
+		size_type dof1 = mf.ind_basic_dof_of_element(cv)[j];
 		for (size_type k = 0; k < N; k++) B[dof1*N + k] += F[dof2*N+k]*(*p);
 	      }
 	  }
@@ -244,9 +244,10 @@ namespace getfem {
   }
 
   template<class MATRM, class MESH_FEM>
-  void old_asm_mass_matrix_on_boundary(MATRM &M, const mesh_im &mim, const MESH_FEM &mf1,
-				       const MESH_FEM &mf2, size_type boundary, dim_type N)
-  {
+  void old_asm_mass_matrix_on_boundary(MATRM &M,
+				       const mesh_im &mim, const MESH_FEM &mf1,
+				       const MESH_FEM &mf2,
+				       size_type boundary, dim_type N) {
     size_type cv, nbd1, nbd2, f;
     dal::bit_vector nn = mf1.convex_index();
     getfem::mesh_region::face_bitset nf;
@@ -255,14 +256,12 @@ namespace getfem {
     pintegration_method pim, pimprec = 0;
     bgeot::pgeometric_trans pgt, pgtprec = NULL;
     pmat_elem_type pme; pmat_elem_computation pmec = 0;
-    // M(0,0) = 1.0;  ??
 
     if (&(mf1.linked_mesh()) != &(mf2.linked_mesh()))
       GMM_ASSERT1(false,
 		  "This assembling procedure only works on a single mesh");
 
-    for (cv << nn; cv != ST_NIL; cv << nn)
-      {
+    for (cv << nn; cv != ST_NIL; cv << nn) {
 	nf = mf1.linked_mesh().region(boundary).faces_of_convex(cv);
 	if (nf.count() > 0) {
 	  pf1 = mf1.fem_of_element(cv); nbd1 = pf1->nb_dof(cv);
@@ -284,10 +283,10 @@ namespace getfem {
 	    
 	      base_tensor::iterator p = t.begin();
 	      for (size_type i = 0; i < nbd2; i++) {
-		size_type dof2 = mf2.ind_dof_of_element(cv)[i];
+		size_type dof2 = mf2.ind_basic_dof_of_element(cv)[i];
 		// cout << "cv = " << cv << " dof2 = " << dof2 << endl;
 		for (size_type j = 0; j < nbd1; j++, ++p) {
-		  size_type dof1 = mf1.ind_dof_of_element(cv)[j];
+		  size_type dof1 = mf1.ind_basic_dof_of_element(cv)[j];
 		  // cout << "dof1 = " << dof1 << " dof2 = " << dof2 << endl;
 		  for (size_type k = 0; k < N; k++)
 		    M(dof1*N + k, dof2*N + k) += (*p);
@@ -300,8 +299,8 @@ namespace getfem {
   }
 
   template<class MATRM, class MESH_FEM>
-  void old_asm_mass_matrix(MATRM &M, const mesh_im &mim, const MESH_FEM &mf1, const MESH_FEM &mf2, dim_type N)
-  {
+  void old_asm_mass_matrix(MATRM &M, const mesh_im &mim, const MESH_FEM &mf1,
+			   const MESH_FEM &mf2, dim_type N) {
     size_type cv, nbd1, nbd2;
     dal::bit_vector nn = mf1.convex_index();
     base_tensor t;
@@ -309,14 +308,12 @@ namespace getfem {
     pintegration_method pim, pimprec = 0;
     bgeot::pgeometric_trans pgt, pgtprec = NULL;
     pmat_elem_type pme; pmat_elem_computation pmec = 0;
-    // M(0,0) = 1.0;  ??
 
     if (&(mf1.linked_mesh()) != &(mf2.linked_mesh()))
       GMM_ASSERT1(false,
 		  "This assembling procedure only works on a single mesh");
 
-    for (cv << nn; cv != ST_NIL; cv << nn)
-      {
+    for (cv << nn; cv != ST_NIL; cv << nn) {
 	pf1 = mf1.fem_of_element(cv); nbd1 = pf1->nb_dof(cv);
 	pf2 = mf2.fem_of_element(cv); nbd2 = pf2->nb_dof(cv);
 	pgt = mf1.linked_mesh().trans_of_convex(cv);
@@ -332,13 +329,11 @@ namespace getfem {
 	// cout << "t = " << t << endl;
       
 	base_tensor::iterator p = t.begin();
-	for (size_type i = 0; i < nbd2; i++)
-	  {
-	    size_type dof2 = mf2.ind_dof_of_element(cv)[i];
+	for (size_type i = 0; i < nbd2; i++) {
+	    size_type dof2 = mf2.ind_basic_dof_of_element(cv)[i];
 	    // cout << "cv = " << cv << " dof2 = " << dof2 << endl;
-	    for (size_type j = 0; j < nbd1; j++, ++p)
-	      {
-		size_type dof1 = mf1.ind_dof_of_element(cv)[j];
+	    for (size_type j = 0; j < nbd1; j++, ++p) {
+		size_type dof1 = mf1.ind_basic_dof_of_element(cv)[j];
 		// cout << "dof1 = " << dof1 << " dof2 = " << dof2 << endl;
 		for (size_type k = 0; k < N; k++)
 		  M(dof1*N + k, dof2*N + k) += (*p);
@@ -439,12 +434,9 @@ namespace getfem {
   }
 
   template<class MAT, class VECT>
-  void old_asm_stiffness_matrix_for_linear_elasticity(MAT &RM,
-						      const mesh_im &mim, 
-						     const mesh_fem &mf, 
-						     const mesh_fem &mfdata, 
-						     const VECT &LAMBDA, const VECT &MU)
-  { //  verifier
+  void old_asm_stiffness_matrix_for_linear_elasticity
+  (MAT &RM, const mesh_im &mim, const mesh_fem &mf, 
+   const mesh_fem &mfdata, const VECT &LAMBDA, const VECT &MU) {
 
     size_type cv, nbd2, N = mf.linked_mesh().dim();
     dal::bit_vector nn = mf.convex_index();
@@ -458,64 +450,55 @@ namespace getfem {
       GMM_ASSERT1(false,
 		  "This assembling procedure only works on a single mesh");
   
-    for (cv << nn; cv != ST_NIL; cv << nn)
-      {
-	pf1 =     mf.fem_of_element(cv); 
-	pf2 = mfdata.fem_of_element(cv); nbd2 = pf2->nb_dof(cv);
-	pgt = mf.linked_mesh().trans_of_convex(cv);
-	pim = mim.int_method_of_element(cv);
-	if (pf1prec != pf1 || pf2prec != pf2 || pgtprec != pgt || pimprec != pim)
-	  {
-	    pme = mat_elem_product(mat_elem_product(mat_elem_grad(pf1),
-						    mat_elem_grad(pf1)), 
-				   mat_elem_base(pf2));
-	    pmec = mat_elem(pme, pim, pgt);
-	    pf1prec = pf1; pf2prec = pf2; pgtprec = pgt; pimprec = pim;
-	  }
-	pmec->gen_compute(t, mf.linked_mesh().points_of_convex(cv), cv);
-	base_tensor::iterator p = t.begin();
+    for (cv << nn; cv != ST_NIL; cv << nn) {
+      pf1 =     mf.fem_of_element(cv); 
+      pf2 = mfdata.fem_of_element(cv); nbd2 = pf2->nb_dof(cv);
+      pgt = mf.linked_mesh().trans_of_convex(cv);
+      pim = mim.int_method_of_element(cv);
+      if (pf1prec != pf1 || pf2prec != pf2 || pgtprec != pgt || pimprec != pim)
+	{
+	  pme = mat_elem_product(mat_elem_product(mat_elem_grad(pf1),
+						  mat_elem_grad(pf1)), 
+				 mat_elem_base(pf2));
+	  pmec = mat_elem(pme, pim, pgt);
+	  pf1prec = pf1; pf2prec = pf2; pgtprec = pgt; pimprec = pim;
+	}
+      pmec->gen_compute(t, mf.linked_mesh().points_of_convex(cv), cv);
+      base_tensor::iterator p = t.begin();
       
-	size_type nbd = mf.nb_dof_of_element(cv);
-
-	for (size_type r = 0; r < nbd2; r++)
-	  {
-	    size_type dof3 = mfdata.ind_dof_of_element(cv)[r];
-	    for (dim_type l = 0; l < N; l++)
-	      for (size_type j = 0; j < nbd; j++)
-		{
-		  size_type dof2 = mf.ind_dof_of_element(cv)[j];
+      size_type nbd = mf.nb_basic_dof_of_element(cv);
+      
+      for (size_type r = 0; r < nbd2; r++) {
+	size_type dof3 = mfdata.ind_basic_dof_of_element(cv)[r];
+	for (dim_type l = 0; l < N; l++)
+	  for (size_type j = 0; j < nbd; j++) {
+	    size_type dof2 = mf.ind_basic_dof_of_element(cv)[j];
 	    
-		  for (dim_type k = 0; k < N; k++)
-		    for (size_type i = 0; i < nbd; i++, ++p)
-		      {
-			size_type dof1 = mf.ind_dof_of_element(cv)[i];
+	    for (dim_type k = 0; k < N; k++)
+	      for (size_type i = 0; i < nbd; i++, ++p) {
+		size_type dof1 = mf.ind_basic_dof_of_element(cv)[i];
 		
-			if (dof1*N + k >= dof2*N + l)
-			  {
-			    RM(dof1*N + k, dof2*N + l) += LAMBDA[dof3] * (*p);
-			    RM(dof2*N + l, dof1*N + k) = RM(dof1*N + k, dof2*N + l);
-			  }
-		
-			if (dof1*N + l >= dof2*N + k)
-			  {
-			    RM(dof1*N + l, dof2*N + k) += MU[dof3] * (*p);
-			    RM(dof2*N + k, dof1*N + l) = RM(dof1*N + l, dof2*N + k);
-			  }
-
-			// cout << "matr elem : " << int(l) << " " << int(j) << " " << int(k) << " " << int(i) << " : " << *p << endl; getchar();
-	      
-			if (l == k && dof1 >= dof2)
-			  for (size_type n = 0; n < N; ++n)
-			    {
-			      RM(dof1*N + n, dof2*N + n) += MU[dof3] * (*p);
-			      RM(dof2*N + n, dof1*N + n) = RM(dof1*N + n, dof2*N + n);
-			    }
-		
-		      }
+		if (dof1*N + k >= dof2*N + l) {
+		  RM(dof1*N + k, dof2*N + l) += LAMBDA[dof3] * (*p);
+		  RM(dof2*N + l, dof1*N + k) = RM(dof1*N + k, dof2*N + l);
 		}
+		
+		if (dof1*N + l >= dof2*N + k) {
+		  RM(dof1*N + l, dof2*N + k) += MU[dof3] * (*p);
+		  RM(dof2*N + k, dof1*N + l) = RM(dof1*N + l, dof2*N + k);
+		}
+		
+		if (l == k && dof1 >= dof2)
+		  for (size_type n = 0; n < N; ++n) {
+		    RM(dof1*N + n, dof2*N + n) += MU[dof3] * (*p);
+		    RM(dof2*N + n, dof1*N + n) = RM(dof1*N + n, dof2*N + n);
+		  }
+		
+	      }
 	  }
-	if (p != t.end()) GMM_ASSERT1(false, "internal error"); 
       }
+      if (p != t.end()) GMM_ASSERT1(false, "internal error"); 
+    }
   }
 
   template<class MAT, class VECT>
@@ -629,13 +612,13 @@ namespace getfem {
 	// cout << "elem matrix " << t << endl;
 	base_tensor::iterator p = t.begin();
 	for (size_type r = 0; r < nbd2; r++) {
-	  size_type dof3 = mfdata.ind_dof_of_element(cv)[r];
+	  size_type dof3 = mfdata.ind_basic_dof_of_element(cv)[r];
 	  for (size_type l = 0; l < N; l++) {
 	    for (size_type i = 0; i < nbd1; i++) {
-	      size_type dof2 = mf.ind_dof_of_element(cv)[i];
+	      size_type dof2 = mf.ind_basic_dof_of_element(cv)[i];
 	      p += l * nbd1;
 	      for (size_type j = 0; j < nbd1; j++, ++p) {
-		size_type dof1 = mf.ind_dof_of_element(cv)[j];
+		size_type dof1 = mf.ind_basic_dof_of_element(cv)[j];
 		if (dof1 >= dof2) { 
 		  RM(dof1, dof2) += A[dof3]*(*p);
 		  RM(dof2, dof1) = RM(dof1, dof2);
@@ -668,7 +651,7 @@ namespace getfem {
 	pf1 =     mf.fem_of_element(cv);
 	pgt = mf.linked_mesh().trans_of_convex(cv);
 	pim = mim.int_method_of_element(cv);
-	size_type nbd = mf.nb_dof_of_element(cv);
+	size_type nbd = mf.nb_basic_dof_of_element(cv);
 	if (pf1prec != pf1 || pgtprec != pgt || pimprec != pim)
 	  {
 	    pme = mat_elem_product(mat_elem_base(pf1), mat_elem_base(pf1));
@@ -680,7 +663,7 @@ namespace getfem {
 
 	for (size_type i = 0; i < nbd; i++)
 	  { 
-	    size_type dof1 = mf.ind_dof_of_element(cv)[i];
+	    size_type dof1 = mf.ind_basic_dof_of_element(cv)[i];
 	    if (vval[i].size() != N) vval[i] = base_vector(N); 
 	    for (size_type k = 0; k < N; k++) (vval[i])[k] = U[dof1*N+k];
 	  }
@@ -711,7 +694,7 @@ namespace getfem {
 	pf1 =     mf.fem_of_element(cv);
 	pgt = mf.linked_mesh().trans_of_convex(cv);
 	pim = mim.int_method_of_element(cv);
-	size_type nbd = mf.nb_dof_of_element(cv);
+	size_type nbd = mf.nb_basic_dof_of_element(cv);
 	if (pf1prec != pf1 || pgtprec != pgt || pimprec != pim)
 	  {
 	    pme = mat_elem_product(mat_elem_grad(pf1), mat_elem_grad(pf1));
@@ -722,7 +705,7 @@ namespace getfem {
 	base_tensor::iterator p = t.begin();
 	for (size_type i = 0; i < nbd; i++)
 	  { 
-	    size_type dof1 = mf.ind_dof_of_element(cv)[i];
+	    size_type dof1 = mf.ind_basic_dof_of_element(cv)[i];
 	    if (vval[i].size() != N) vval[i] = base_vector(N); 
 	    for (size_type k = 0; k < N; k++) (vval[i])[k] = U[dof1*N+k];
 	  }
@@ -886,10 +869,10 @@ void init_mesh_im(getfem::mesh_im &mim, bool use_exact_im=true) {
   }
 }
 
-void comp_mat(const sparse_matrix_type& M1, const sparse_matrix_type& M2)
-{
+void comp_mat(const sparse_matrix_type& M1, const sparse_matrix_type& M2) {
   scalar_type d = 0;
-  scalar_type mx = 1e-200; /* avoid triggering an FPE for bound assembly when there is no boundary */
+  scalar_type mx = 1e-200; /* avoid triggering an FPE for bound assembly */
+                           /* when there is no boundary                  */
   sparse_vector_type r(gmm::mat_ncols(M1));
   for (size_type i = 0; i < gmm::mat_nrows(M1); ++i) {
     mx = std::max(mx,gmm::vect_norminf(gmm::mat_const_row(M1,i)));
@@ -907,7 +890,8 @@ void comp_mat(const sparse_matrix_type& M1, const sparse_matrix_type& M2)
       gmm::copy(gmm::mat_const_row(M2,i),r2);    
       cout << "\nrow(" << i+1 << "),\nM1=" << r1 << "\nM2=" << r2 << endl;
       fail_cnt++;
-      cout << " FAILED !";
+      GMM_ASSERT1(false, "Failed ! ");
+      // cout << " FAILED !";
       break;
     }
   }
@@ -915,8 +899,7 @@ void comp_mat(const sparse_matrix_type& M1, const sparse_matrix_type& M2)
   cout << " ---> difference between assemblies: " << d / mx << "\n\n";
 }
 
-void comp_vec(const base_vector& V1, const base_vector& V2)
-{
+void comp_vec(const base_vector& V1, const base_vector& V2) {
   scalar_type mx = std::max(gmm::vect_norminf(V1),gmm::vect_norminf(V2));
   base_vector dv = V2;
   gmm::add(gmm::scaled(V1, -1.0),dv);
@@ -1163,12 +1146,14 @@ void
 run_tests(getfem::mesh_im &mim, 
 	  getfem::mesh_fem& mf, getfem::mesh_fem& mfq,
 	  getfem::mesh_fem& mfd, getfem::mesh_fem& mfdq,
-	  bool do_new, bool do_old, const std::vector<bool>& do_what, unsigned nloop, unsigned nloop_bound) {
+	  bool do_new, bool do_old, const std::vector<bool>& do_what,
+	  unsigned nloop, unsigned nloop_bound) {
   size_type Ndim = mf.linked_mesh().dim();
   bool do_new2 = do_new;
   base_vector V1q(Ndim*mf.nb_dof()), V2q(mfq.nb_dof());
   base_vector V1(mf.nb_dof()), V2(mf.nb_dof());
-  sparse_matrix_type M1(mfq.nb_dof(),mfq.nb_dof()), M2(mfq.nb_dof(),mfq.nb_dof());
+  sparse_matrix_type M1(mfq.nb_dof(),mfq.nb_dof());
+  sparse_matrix_type M2(mfq.nb_dof(),mfq.nb_dof());
 
   chrono c;
     
@@ -1324,6 +1309,10 @@ run_tests(getfem::mesh_im &mim,
     cout << "done " << c << endl;
   }
   if (do_new) {
+    // (sparse ws 99ms), (full 89), (sparse rs) 90ms)
+    cout << "mfq.nb_dof() = " << mfq.nb_dof() << endl;
+    // getfem::base_matrix M3(mfq.nb_dof(), mfq.nb_dof());
+    // 
     cout << "laplacian, Q=1, new way [" << nloop << " times] .." << flushy;
     c.init();
     for (size_type cnt = 0; cnt < nloop; ++cnt) {
@@ -1331,6 +1320,7 @@ run_tests(getfem::mesh_im &mim,
       getfem::asm_stiffness_matrix_for_laplacian(M2, mim, mf, mfd, A); 
       c.toc(); cout << "#" << flushy;
     }
+    // gmm::copy(M3, M2);
     cout << "done " << c << endl;
   }
   if (do_old && do_new) comp_mat(M1,M2);
@@ -1427,8 +1417,9 @@ run_tests(getfem::mesh_im &mim,
     cout << "done " << c << endl;
   }
   if (do_new) {
-    cout << "linear elasticity, Q=" << Ndim<<", new way [" << nloop << " times] .." << flushy;
+    cout << "linear elasticity, Q=" << Ndim << ", new way [" << nloop << " times] .." << flushy;
     c.init();
+    // (sparsec ws : 1:29ms 2:299ms), (sparsec rs : 1:29ms 2:250ms), (full : 1:30ms 2:209ms)
     for (size_type cnt = 0; cnt < nloop; ++cnt) {
       gmm::clear(M2); c.tic();
       getfem::asm_stiffness_matrix_for_linear_elasticity(M2, mim, mfq, mfd, A, A2);
